#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	Vasilis.Vlachoudis@cern.ch
# Date:	20-Aug-2015

from __future__ import absolute_import
from __future__ import print_function

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

__name__ = _("Random")
import random
from ToolsPage import Plugin

try:
	import tkMessageBox
except ImportError:
	import tkinter.messagebox as tkMessageBox

#import math
#from bmath import Vector
#from CNC import CW,CCW,CNC,Block


#==============================================================================
# Tile replicas of the selected blocks
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Generate replicas of selected code")

	def __init__(self, master):
		Plugin.__init__(self, master, "Random")
		self.icon  = "randomize"
		self.group = "CAM"
		self.variables = [
			("name",      "db",    "", _("Name")),
			("randx",        "mm",  5.0, "Rand x"),
			("randy",        "mm",  5.0, "Rand y")
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def execute(self, app):
		# Get selected blocks from editor
		blocks = app.editor.getSelectedBlocks()
		if not blocks:
			app.editor.selectAll()
			blocks = app.editor.getSelectedBlocks()

		if not blocks:
			tkMessageBox.showerror(_("Tile error"),
				_("No g-code blocks selected"))
			return

		try:
			dx = self.fromMm("randx")
		except:
			dx = 0.0

		try:
			dy = self.fromMm("randy")
		except:
			dy = 0.0

		pos = blocks[-1]	# insert position


		pos += 1
		# clone selected blocks
		undoinfo = []	# FIXME it should be only one UNDO
		
		for bid in blocks:
			undoinfo.append(app.gcode.cloneBlockUndo(bid, pos))
			newblocks = []
			newblocks.append((pos,None))
			pos += 1
			app.addUndo(undoinfo)
			x = random.uniform(-dx, dx)
			y = random.uniform(-dy, dy)
			app.gcode.moveLines(newblocks, x, y)
			
		allBlocks = app.gcode.blocks	
		for bid in blocks:
			block = allBlocks[bid]
			if not block.name() in ("Header", "Footer"):
				block.enable = False
		app.refresh()
		app.setStatus(_("Tiled selected blocks"))



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author: Mario Basz
#mariob_1960@yahoo.com.ar
# Date: 20 may 2018
# A special thanks to Vasilis Viachoudis, Filippo Rivato and Buschhardt 
#This plugin is based on a variation
# of yours Driller plugin and My_Plugin example.

__author__ = "Mario S Basz"
__email__  = "mariob_1960@yaho.com.ar"

__name__ = _("Trochoidcut")
__version__= "1.3"

import os.path
import re
import math
from collections import OrderedDict
from CNC import CNC,Block
from ToolsPage import Plugin
from bmath import pi
from math import pi, sqrt, sin, cos, asin, acos, atan2, hypot, degrees, radians, copysign, fmod, tan, ceil

#==============================================================================
# Create trochoids along selected block
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Create trochoids along selected blocks")
	def __init__(self, master):
		Plugin.__init__(self, master, "Trochoidal 3D")
		self.icon  = "trochoidal"
		self.group = "CAM"

		self.variables = [
			("name",          "db",    "",    _("Name")),
            ("diam" ,"float", "", _("Trochoid Cut Diameter")),
			("endmill",   "db" ,    "", _("End Mill"), "If Empty chooses, End Mill loaded"),
			("ae", "mm", 0.30, _("Trochoids Advance")),
			("TypeSplice","Warpedarc,Splices,Circular one side rectified,Circular both sides rectified,Straight,Cut",\
			 "Warpedarc", _("Type of Splice"),"Currently Splices works very slow"),
			("direction","inside,outside,on (3d Path)" , "inside", _("Direction")),
			("offset",   "float",                   0.0, _("Additional offset distance"), " Only for In or On direction"),
			("adaptative",  "bool",                   1,   _("Adaptative"), "Generate path for adaptative trochoids in the corners. Only for In or On direction"),
			("overcut",  "bool",                      0, _("Overcut"), " Only for In or On direction"),
			("targetDepth",  "mm",                    -1, _("Target Depth"), " Only for In or On direction"),
			("depthIncrement",  "mm",                  1, _("Depth Increment"), " Only for In or On direction"),
			("cw"    ,    "bool" ,    True, _("Clockwise")),
#			("helicalangle", "int" ,  25, _("Helical descent angle")),
			("rpm",          "int" ,    12000, _("RPM")),
			("feed"		, "int", 1200	, _("Feed")),
			("zfeed"		, "int", ""	, _("Plunge Feed")),
			("splicesteps",       "int", 12 , _("splice steps every 360 degrees"), "Only for Splices Type of Splice"),
#			("tabsnumber",  "mm",                      1, _("Number of Tabs 0 = Not Tabs")),
#			("tabsWidth",  "mm",                       1, _("Tabs Diameter")),
#			("tabsHeight",  "mm",                       1, _("Tabs Height")),

#			("VerticalResolution"     ,  "mm" ,  0.15, _("Resolution or Vertical Step")),
#			("variableRPM"		, "bool",True	, _("Variable RPM")),
#			("S_z",       "int",  "", _("RPM For Descent Cut")),
#			("S_xy",       "int",  "", _("RPM Trochoidal Cutting")),
#			("variableFeed"		, "bool",True	, _("Variable Feed")),
#			("K_Z" ,       "float", 1.00, _("Feed rate Multiplier Z")),
#			("K_XY" ,       "float", 1.00, _("Feed rate Multiplier XY")),
#			("TargetDepth",   "mm",  0.0, _("Target Depth")),
	#		("_3D"		, "bool",False	, _("Cut accord 3D Path")),
#			("VerticalClimbing"	, "bool",False	, _("Vertical Climbing Trochoid")),
#			("AllowSurface"		, "bool",False	, _("Trochoid on Z Surface")),
#			("MinTrochDiam",       "float", 6.0 , _("Minimal trochoid in % tool")),
#			("OnlyGline"		, "bool",False	, _("Only go to Trochoid Center")),
#			("FirstPoint"		, "bool",False	, _("First Point Problem")),
#			("Dwell",      "float",  0.0, _("Dwell time, 0 means None")),
#			("Peck",          "mm",    0.0,   _("Peck, 0 meas None")),
#			("manualsetting", "bool", False	 , _("----- Manual setting ------")),
 #           ("helicalDiam" ,"float", 7.0, _("Helical Descent Diameter")),
#			("minimfeed",   "int" ,   "", _("Minimum Adaptative Feed")),
		]
		self.buttons.append("exe")
		self.help= '''It generates the trochoidal cutting mode on the selected block, even at different heights.
Currently, it only works by selecting only one block (a solution is to join the blocks).
In or Out generates the route and on it generates trochoids.
'''

# calculate separation between centers of trochoidsegments -----------------------------------------------------
#	def information(self, xyz):
#		return 0
	# ----------------------------------------------------------------------
	def update(self):
		self.master.cnc()["diam"] = self.fromMm("diam")
	# ----------------------------------------------------------------------
	def came_back(self, current, old):
		A= current[0]
		B= current[1]
		oldA=old[0]
		oldB=old[1]
	#	if A[0]==oldB[0] and A[1]==oldB[1] and A[2]==oldB[2] \
		if	B[0]==oldA[0] and B[1]==oldA[1] and B[2]==oldA[2]:
			control=1
		else:
			control=0
		return control

	def center_distance(self, xyz, atot):
#		atot = self.fromMm("ae")
#		ae=atot
		xadd=yadd=zadd=0
		#xyz:
		A = xyz[0]
		B = xyz[1]
		#segmlength = self.calcSegmentLength(xyz)
		segmlength = math.sqrt((B[0]-A[0])**2 + (B[1]-A[1])**2 + (B[2]-A[2])**2)
		segmlength_xy= math.sqrt((B[0]-A[0])**2 + (B[1]-A[1])**2)

		xlength = B[0]-A[0]
		ylength = B[1]-A[1]
		zlength = B[2]-A[2]
		# on ascending ramp keep the ae value on xy plane	
		if xlength !=0 or ylength!=0 and zlength >0:
			atot *= segmlength/segmlength_xy

		if segmlength >0:
			xadd = atot*(xlength/segmlength)
			yadd = atot*(ylength/segmlength)
			zadd = atot*(zlength/segmlength)
			ae= math.sqrt(xadd**2+yadd**2)

		return xadd, yadd, zadd, atot, ae # 

	# Calc subsegments -----------------------------------------------------
	def calcSegmentLength(self, xyz):
		if xyz:
			A = xyz[0]
			B = xyz[1]
			seglength_x = B[0]-A[0]
			seglength_y = B[1]-A[1]
			seglength_z = B[2]-A[2]
#			seglength_xyz= math.sqrt(seglength_x**2 + seglength_y**2 + seglength_z**2)
#			results= seglength_x, seglength_y, seglength_z, seglength_xyz
#			results = seglength_xyz
#			return results  
			return math.sqrt(seglength_x**2 + seglength_y**2 + seglength_z**2)
		else:
			return 0
	# ----------------------------------------------------------------------
	#Extract all segments from commands ------------ -----------------------
	def extractAllSegments(self, app,selectedBlock):
		allSegments = []
		allBlocks = app.gcode.blocks

		for bid in selectedBlock:
			bidSegments = []
			block = allBlocks[bid]
			if block.name() in ("Header", "Footer"): 
				continue
			#if not block.enable : continue
			app.gcode.initPath(bid)
			for line in block:
				try:
					cmd = app.cnc.breakLine(app.gcode.evaluate(app.cnc.compileLine(line)))
				except:
					cmd = None

				if cmd:
					app.cnc.motionStart(cmd)
					xyz = app.cnc.motionPath()
					app.cnc.motionEnd()

					if xyz:
						#exclude if fast move or z only movement
						G0 =('g0' in cmd) or ('G0' in cmd)
						Zonly = (xyz[0][0] == xyz[1][0] and xyz[0][1] == xyz[1][1])
						exclude = Zonly

						#save length for later use
						segLength = self.calcSegmentLength(xyz)

						if len(xyz) < 3:
							bidSegments.append([xyz[0],xyz[1],exclude,segLength])
						else:
							for i in range(len(xyz)-1):
								bidSegments.append([xyz[i],xyz[i+1],exclude,segLength])
			#append bidSegmentes to allSegmentes
			allSegments.append(bidSegments)

		#Disabled used block
		for bid in selectedBlock:
			block = allBlocks[bid]
			if block.name() in ("Header", "Footer"): continue
			block.enable = False

		return allSegments

	# ----------------------------------------------------------------------
	def execute(self, app):
#		ae = self.fromMm("ae")
		if self["splicesteps"] =="" or self["splicesteps"]<4:
			steps=4/(2*pi)
		else:
			steps=self["splicesteps"]/(2*pi)

#		manualsetting = self["manualsetting"]
		manualsetting = 1
#=========== Converted to comment and changed for current compatibility ==============================
#		cutradius = CNC.vars["trochcutdiam"]/2.0
		cutradius = self["diam"]/2.0
#		cutradius = CNC.vars["trochcutdiam"]/2.0
#=========================================================================================
		zfeed = CNC.vars["cutfeedz"]
		feed =CNC.vars["cutfeed"]
		minimfeed =CNC.vars["cutfeed"]
		if manualsetting:
			if self["diam"]:
				cutradius = self["diam"]/2.0

			if self["zfeed"] and self["zfeed"]!="":
				zfeed = self["zfeed"]

#			if self["minimfeed"] and self["minimfeed"]!="":
#				minimfeed = min (self["minimfeed"],feed)

			if self["feed"] and self["feed"]!="":
				feed = self["feed"]
	
			if self["endmill"]:
				self.master["endmill"].makeCurrent(self["endmill"])



#		radius = CNC.vars["cutdiam"]/2.0
#		radius = self["diam"]/2.0
		toolRadius = CNC.vars["diameter"]/2.0
		radius = max(0,cutradius-toolRadius)
		oldradius=radius
#-----------------------------------------------------------
#		helicalRadius = self["helicalDiam"]/2.0
#		if helicalRadius=="":
#			helicalRadius=radius
#		else:
#			helicalRadius=max(0,helicalRadius- toolRadius)
		helicalRadius=radius
#-----------------------------------------------------------

#		helicalRadius=min(0.99*toolRadius,helicalRadius)
#		if radius!=0:
#			helicalRadius= min(helicalRadius,radius)
		helicalPerimeter=pi*2.0*helicalRadius		
	
	#	helicalangle = self["helicalangle"]
	#	if helicalangle>89.5:
	#		helicalangle=89.5
	#	if helicalangle<0.01:
	#		helicalangle=0.01
	#	downPecking=helicalPerimeter*tan(radians(helicalangle))

		cw = self["cw"]
		surface = CNC.vars["surface"]
#=========== Converted to comment and changed for current compatibility ==============================
#		zbeforecontact=surface+CNC.vars["zretract"]
#		zbeforecontact=surface+CNC.vars["zretract"]
#		hardcrust = surface - CNC.vars["hardcrust"]
#		feedbeforecontact = CNC.vars["feedbeforecontact"]/100.0
#		hardcrustfeed = CNC.vars["hardcrustfeed"]/100.0

		zbeforecontact=surface
		zbeforecontact=surface
		hardcrust = surface
		feedbeforecontact = zfeed
		hardcrustfeed = feed
#=====================================================================================================

		t_splice = self["TypeSplice"]
		dtadaptative = 0.0001
		adaptativepolice=0
#		minimradius = min(radius, toolRadius*self["MinTrochDiam"]/(100))
#		minimradius = min(radius, toolRadius*self["MinTrochDiam"]/(100))
#		minimradius = min(radius, toolRadius*CNC.vars["mintrochdiam"]/(100))
		atot = self.fromMm("ae")
#		spiral_twists=(radius-helicalRadius)/atot#<<spiral ae smaller than ae (aprox 50%)
#		if (radius-helicalRadius)%atot: spiral_twists=1+(radius-helicalRadius)//atot
		spiral_twists=ceil(radius-helicalRadius)/atot#<<spiral ae smaller than ae (aprox 50%)

		rpm = self["rpm"]

		downPecking=helicalPerimeter*zfeed/feed
		helicalangle=degrees(atan2(downPecking,helicalPerimeter))

#		steps=self["splicesteps"]/2*pi

#		K_Z = self["K_Z"]
#		if K_Z == "":
#			K_Z = 1.0
#		K_XY = self["K_XY"]
#		if K_XY == "": 
#			K_XY = 1.0
#		s_z = self["S_z"]
#		s_xy = self["S_xy"]
#		xyfeed = CNC.vars["cutfeed"]
#		zfeed *= K_Z
#		xyfeed *=K_XY		


		# Get selected blocks from editor
#	def trochprofile_bcnc(self, cutDiam=0.0, direction=None, offset=0.0, overcut=False,adaptative=False, adaptedRadius=0.0, tooldiameter=0.0, name=None):

#		app.trochprofile_bcnc(trochcutdiam, direction, self["offset"], self["overcut"], self["adaptative"], cornerradius, CNC.vars["diameter"], name) #<< diameter only to information
	#	cornerradius = (cutradius - CNC.vars["diameter"]/2.0
		direction=self["direction"]
		if direction!="on (3d Path)":
			targetDepth=self["targetDepth"]
			depthIncrement=self["depthIncrement"]
#			tabsnumber=self["tabsnumber"]
#			tabsWidth=self["tabsWidth"]
#			tabsHeight=self["tabsHeight"]
			tabsnumber=tabsWidth=tabsHeight=0

			app.trochprofile_bcnc(2*cutradius, direction,self["offset"], self["overcut"], self["adaptative"], radius, CNC.vars["diameter"],\
				targetDepth, depthIncrement, tabsnumber, tabsWidth, tabsHeight)
			app.refresh()
#		app.editor.selectAll()

		selBlocks = app.editor.getSelectedBlocks()
#		if not selBlocks:
#			app.editor.selectAll()
#			selBlocks = app.editor.getSelectedBlocks()

		if not selBlocks:
			app.setStatus(_("Trochoid abort: Please select some path"))
			return 
		#Check inputs
		if cutradius <= toolRadius:
				app.setStatus(_("Trochoid Cut Diameter has to be greater than End mill"))
				return

		if helicalRadius <= 0.0:
				app.setStatus(_("Helical Descent Diameter has to be greater than End mill"))
				return

		if feed <= 0:
				app.setStatus(_("Feed has to be greater than 0"))
				return

		if zfeed <= 0:
				app.setStatus(_("Plunge Feed has to be greater than 0"))
				return

		if minimfeed <= 0:
				app.setStatus(_("Minimum Adaptative Feed has to be greater than 0"))
				return

		#Get all segments from gcode
		allSegments = self.extractAllSegments(app,selBlocks)

		#Create holes locations
#		allHoles=[]
		for bidSegment in allSegments:
			if len(bidSegment)==0:
				continue
		blocks = []
#		n = self["name"]
#		if not n or n=="default": n="Trochoidal_3D"
#		newname = Block.operationName(path.name)
		n="Troch3d"
		tr_block = Block(n)
		phi=oldphi=0# oldadaptativephi=0
		oldsegm=[[0,0,0],[0,0,0]]

#		segments ---------------------------------------------
		for idx, segm in enumerate(bidSegment):
			if idx >= 0:
				if cw:
					u = 1
					arc = "G2"
				else:
					u = -1
					arc = "G3"				
#		////////////---------------------------------------------------------------------
#		information: ---------------------------------------------------------------------
				segLength = self.calcSegmentLength(segm)
	#			    ---------------------------------------------
	#				tr_block.append("(seg length "+str(round(segLength,4))+" )")
	#				-----------------------------------------------------------------------------
	#				////////----------------------------------------------------------------------
				if idx == 0:
#					tr_block.append("(--------------   PARAMETERS   ------------------------)")
					tr_block.append("(Cut diam "+str( cutradius*2 )+" (troch "+str(radius*2.0)+"+End mill "+str(toolRadius*2.0)+" ) Advance "+str(atot)+" )")
#					tr_block.append("(Cut diam "+str(CNC.vars["trochcutdiam"])+" (troch "+str(radius*2.0)+" + End mill " + str(toolRadius*2.0)+" ) Advance "+str(atot)+" )")
#					tr_block.append("(Min troch "+str(int(CNC.vars["mintrochdiam"]))+"%  = "+str(minimradius*2.0)+"mm , min cut diam "+str(2*(minimradius+toolRadius))+"mm )")
					tr_block.append("(Feed "+str(feed)+" Plunge feed "+ str(zfeed)+" )")
					#tr_block.append("(Helical diam "+str(round((helicalRadius+toolRadius)*2,2))+" ( helical diam "+str(helicalRadius*2.0)+"+End mill "+str(toolRadius*2.0)+" )")
					tr_block.append("(Helical descent angle " + str(round(helicalangle,2)) +" cut diam " + str(round(helicalRadius*2.0,3))+"  drop by lap "\
										+ str(round(downPecking,2)) + " )")
					tr_block.append("(--------------------------------------------------)")
					tr_block.append("(M06 T0 "+str(toolRadius*2.0)+" mm)")
					tr_block.append("M03")
					tr_block.append("S "+str(rpm))
					tr_block.append("F "+str(feed))
#					phi = atan2(segm[1][1]-segm[0][1], segm[1][0]-segm[0][0])
#					oldphi=phi #<< declare initial angle
#					l = self.pol2car(radius, phi+radians(90*u))
#					r = self.pol2car(radius, phi+radians(-90*u))
#					B = segm[1][0],segm[1][1],segm[1][2]
#					bl = self.pol2car(radius, phi+radians(90*u), B)
#					br = self.pol2car(radius, phi+radians(-90*u), B)
					tr_block.append("( Seg: "+str(idx)+"   length "+str(round(segLength,4))+"  phi "+str(round(degrees(phi),2))+" )")#+ "  oldphi  "+str(round(oldphi*57.29,2))+"   )")
					tr_block.append("(Starting point)")
					if (round(segm[1][1]-segm[0][1],4)==0 and round(segm[1][0]-segm[0][0],4)==0):
						phi=1234567890
						tr_block.append("(The original first movement is vertical)")
					else:
						tr_block.append("(The original first movement is not vertical)")
					tr_block.append(CNC.zsafe())
#					tr_block.append("g0 x "+str(B[0])+" y"+str(B[1])+" )")#" z "+str(B[2])+" )")
#							tr_block.append(arc+" x"+str(bl[0])+" y"+str(bl[1])+" R "+str(radius/2.0)+" z"+str(B[2]))
#					tr_block.append(arc+" x"+str(br[0])+" y"+str(br[1])+" i"+str(r[0]/2.0)+" j"+str(r[1]/2.0))	#<< as cutting					
#					tr_block.append(("g1 x "+str(br[0])+" y"+str(br[1])+" z"+str(B[2])))
#						tr_block.append(arc+" x"+str(bl[0])+" y"+str(bl[1])+" i"+str(l[0])+" j"+str(l[1]))						
#						tr_block.append(arc+" x"+str(br[0])+" y"+str(br[1])+" i"+str(r[0])+" j"+str(r[1])+" z"+str(round(B[2],5)))	#<< as cutting
#					if t_splice=="Circular both sides rectified":
#						tr_block.append(arc+" x"+str(bl[0])+" y"+str(bl[1])+" i"+str(-r[0])+" j"+str(-r[1]))						
					tr_block.append("(--------------------------------------------------)")

#						tr_block.append(CNC.grapid(br[0],br[1],B[2]))
#						tr_block.append(CNC.zsafe()) 			#<<< Move rapid Z axis to the safe height in Stock Material
#						tr_block.append(CNC.zenter(surface)) # <<< TROCHOID CENTER
#						tr_block.append(CNC.grapid(segm[1][0],segm[1][1],segm[1][2]))
#						tr_block.append(CNC.zbeforecontact()) # <<< TROCHOID CENTER
#						tr_block.append(CNC.xyslowentersurface(0,-45.0)) # <<< TROCHOID CENTER
#						tr_block.append(("g0 z "+str(zbeforecontact)))
#				tr_block.append("( new segment begins )")
	#					distance to trochoid center

					# if there is movement in xy plane phi calculate
				if (segm[1][1]-segm[0][1]!=0 or segm[1][0]-segm[0][0]!=0):
					phi = atan2(segm[1][1]-segm[0][1], segm[1][0]-segm[0][0])
#					On surface
#					if segm[0][2]>zbeforecontact and segm[1][2]>zbeforecontact:
					if segm[0][2]>surface and segm[1][2]>surface:
						tr_block.append("(Seg: "+str(idx)+" length "+str(round(segLength,4))+" phi "+str(round(degrees(phi),2))+ " On Surface)" )
						tr_block.append(CNC.grapid(segm[1][0],segm[1][1],segm[1][2]))
					else:
						tr_distance = self.center_distance(segm,atot)
						A = segm[0][0],segm[0][1],segm[0][2]
						d=segLength
						ae = tr_distance[4]
	#				////////////---------------------------------------------------------------------
	#				information: ---------------------------------------------------------------------
						adv = tr_distance[3] #<<< 
						ap = tr_distance[2] # << =zadd
	#			    ---------------------------------------------
						tr_block.append("(-----------------------------------------)")
						control_cameback = self.came_back(segm, oldsegm)
						if control_cameback:
							tr_block.append("(-------------> Came back !! <------------- )")#+str(control_cameback)+" )")
	#						tr_block.append("( old  Ax "+str(round(oldsegm[0][0],3))+" Ay "+str(round(oldsegm[0][1],3))+" Bx "+ str(round(oldsegm[1][0],3))+" By "+ str(round(oldsegm[1][1],3))+" )")
	#						tr_block.append("( curr Ax "+str(round(segm[0][0],3))+" Ay "+str(round(segm[0][1],3))+" Bx "+ str(round(segm[1][0],3))+" By "+ str(round(segm[1][1],3))+" )")
						if round(segLength,5) <= dtadaptative:
							adaptativepolice+=1.0
							tr_block.append("(Seg "+str(idx)+" adaptativepolice " +str(adaptativepolice)+" length "+str(round(segLength,5))+" )")
#	///////////	Trochoid method //////////////////////////////////////////////////////////////////////////////
						if adaptativepolice==0 or adaptativepolice >2.5:

							tr_block.append("( Seg: "+str(idx)+" phi "+str(round(degrees(phi),2))+ " oldphi "+str(round(degrees(oldphi),2))+" length "+str(round(segLength,5))+" )")
							tr_block.append("(ae: "+str(round(ae,5))+" dz: "+str(round(ap,4))+"adv: "+str(round(adv,4))+" )")
		#					tr_block.append("( Bx "+str(round(segm[1][0],2))+ " By "+ str(round(segm[1][1],2)))
		#					-----------------------------------------------------------------------------
		#					////////----------------------------------------------------------------------
							if control_cameback:
		#						adaptativepolice+=0.5
								B = segm[1][0],segm[1][1],segm[1][2]
		#								tr_block.append(CNC.gline(segm[1][0],segm[1][1],segm[1][2]))
								t_splice="came_back"
#								tr_block.extend(self.trochoid(t_splice,A,B,minimradius,radius,oldphi,phi,cw))
								tr_block.extend(self.trochoid(t_splice,A,B,0.0,radius,oldphi,phi,cw))
								tr_block.append("F "+ str(feed))
								t_splice = self["TypeSplice"]

							else:
								# from POINT A -- to ---> POINT B
								if segLength<=adv:
									tr_block.append("(Only one trochoid, oldphi "+str(round(degrees(oldphi),2))+" )")
									tr_block.extend(self.trochoid(t_splice,A,B,oldradius,radius,oldphi,phi,cw))
								while d >adv:
			#					first trochoid
			#						tr_block.append("d "+ str(d))
									B = A[0]+tr_distance[0], A[1]+tr_distance[1], A[2]+tr_distance[2]
								# intermediates points = trochoids points 
			#						tr_block.append(CNC.gline(B[0],B[1],B[2])) # <<< TROCHOID CENTER
		#							tr_block.extend(self.trochoid(A,B,radius,phi,oldphi,cw))

									tr_block.append("(distance to end segment "+str(round(d,4))+" )")
									tr_block.extend(self.trochoid(t_splice,A,B,oldradius,radius,oldphi,phi,cw))
									A=B
									d-=adv
									oldphi=phi
			#					last point
								if  B[0] != segm[1][0] or B[1] != segm[1][1] or B[2] != segm[1][2]:
									B = segm[1][0],segm[1][1],segm[1][2]
			#						tr_block.append(CNC.gline(B[0],B[1],B[2]))  # <<< TROCHOID CENTER
									tr_block.append("(---last trochoid, distance to end segment "+str(round(d,4))+" ---)")
									tr_block.extend(self.trochoid(t_splice,A,B,oldradius,radius,phi,phi,cw))
							adaptativepolice=0

		#	///////	Adapative method //////////////////////////////////////////////////////////////////////////////////////////////////////////
#						if oldphi==3600:
						else:
							if adaptativepolice==1:
								#goes to de two warning movements 
								lastphi=oldphi
								tr_block.append("( Alarm "+ str(adaptativepolice)+"  Seg: "+str(idx)+" phi " + str(round(degrees(phi),2))\
												 + "oldphi "+str(round(degrees(oldphi),2))+ " )")
	#							difangle=(phi-oldadaptativephi)
	#							tr_block.append("(dif angle:"+str(round(difangle,4))+" )")
	#							oldadaptativephi=oldphi=phi
								# round(difangle,5)==round(pi,5):
							elif adaptativepolice==2:
								phi=lastphi
								if control_cameback:# abs(round(difangle,6)) == (round(pi,6)):
									tr_block.append("(Starts adaptative trochoids"+" adaptativepolice "+str(adaptativepolice) )
									adaptativepolice +=0.5
							elif adaptativepolice==2.5:
#								tr_block.append("(-----------------------------------------)")
#								adaptradius=minimradius
								tr_block.append("(Adaptative Seg: "+str(idx)+"   length "+str(round(segLength,5))+"  phi "+str(round(degrees(phi),2))\
												+" oldphi "+str(round(degrees(oldphi),2))+" )")
#								tr_block.append("( Ax "+str(round(segm[0][0],2))+ " Ay "+ str(round(segm[0][1],2)))

#								tr_block.append(CNC.gline(segm[1][0],segm[1][1],segm[1][2]))
								# from POINT A -- to ---> POINT B
#								if adaptativepolice==1:
								tr_distance = self.center_distance(segm,atot/3.0) #<<< short advanc distances 

								A = segm[0][0],segm[0][1],segm[0][2]
								d=segLength
								ae = tr_distance[4]
								adv = tr_distance[3] #<<< 
								d-=adv
								while d >0:#adv:
			#					first trochoid
									if d!=segLength-adv:
										oldphi=phi
			#						tr_block.append("d "+ str(d))
									B = A[0]+tr_distance[0], A[1]+tr_distance[1], A[2]+tr_distance[2]
									#------------------------------
									# adaptradius= a*d + minimradius
									# if d=0 : adaptradius=minimradius
									# if d=seglength : adaptradius=radius
#									a=(radius-minimradius)/segLength
#									adaptradius=a*d+minimradius
									a=radius/segLength
									adaptradius=(self.roundup(a*d,4))#+minimradius
												#------------------------------
									if t_splice!="Splices":
										t_splice="Warpedarc"
									tr_block.append("(from trochoid distance to end segment "+str(round(d,4))+" )")
									tr_block.append("(adaptradius "+ str(round(adaptradius,4))+" radius " + str(radius)+" )")
#									tr_block.append("F "+ str(feed*adaptradius//radius))
									tr_block.append("F "+ str(minimfeed+(feed-minimfeed) *adaptradius//radius))
									if adaptradius>0.0:
										tr_block.extend(self.trochoid(t_splice,A,B,oldradius,adaptradius,oldphi,phi,cw))
									else:
										tr_block.append("(R= "+str(adaptradius)+ "not sent )")
#										tr_block.append("G1 x"+str(round(B[0],4))+" y "+str(round(B[1],4))+" z "+str(round(B[2],4)))
									A=B
									d-=adv
									oldradius=adaptradius
#										oldadaptativephi=0
			#REVISAR, A COMENTADO
			#					last point
					#			d=0
					#			oldradius=adaptradius
					#			adaptradius=minimradius
					#			if  B[0] != segm[1][0] or B[1] != segm[1][1] or B[2] != segm[1][2]:
					#				B = segm[1][0],segm[1][1],segm[1][2]
			#						tr_block.append(CNC.gline(B[0],B[1],B[2]))  # <<< TROCHOID CENTER
					#				tr_block.append("(last trochoid, from trochoid distance to end segment "+str(round(d,4))+" )")
					#				tr_block.append("(adaptradius "+ str(adaptradius)+" )")
					#				tr_block.append("F "+ str(feed*adaptradius//radius))
					#				tr_block.extend(self.trochoid(t_splice,A,B,oldradius,adaptradius,phi,phi,cw))

								adaptativepolice=0
								tr_block.append("(Adaptative Completed)")
								tr_block.append("F "+ str(feed//3))

#								if adaptativepolice>1:
							t_splice = self["TypeSplice"]
#							adaptativepolice=0
							oldradius=radius
						oldsegm=segm
					oldphi=phi
#	///////	Vertical movement ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				elif idx!=0:	
					tr_block.append("(Seg: "+str(idx)+" length "+str(round(segLength,4))+" phi "+str(round(degrees(phi),2))+" oldphi "+str(round(degrees(oldphi),2))+" )" )
					tr_block.append("(Helical descent")
#						descent
					A=segm[0][0],segm[0][1],segm[0][2]
					if segm[0][2] > segm[1][2]:
						if segm[0][2] >zbeforecontact:# and segm[1][2]<=surface:
							if segm[1][2]<=zbeforecontact:
								B = segm[1][0],segm[1][1],max(segm[1][2],zbeforecontact)
								tr_block.append("(Rapid helical to z before contact "+"helicalRadius "+str(helicalRadius)+" )")
								if idx==1 and oldphi==1234567890:
									tr_block.append("g0 x "+str(B[0])+" y"+str(B[1])+" )")#" z "+str(B[2])+" )")
								tr_block.extend(self.helical(A,B,helicalRadius,phi,u))

	#						Instead of decreasing the speed, to avoid the jerkl, decrease the drop by lap
						if segm[0][2] >surface:# and segm[1][2]<=surface:
							if segm[1][2]<=surface:
								tr_block.append("(Slow helical to surface )" )
								A=A[0],A[1],zbeforecontact
								d=A[2]-surface
								adv=downPecking * feedbeforecontact
								while d > adv:
									B = segm[1][0],segm[1][1],max(segm[1][2],A[2]-adv)
									tr_block.extend(self.helical(A,B,helicalRadius,phi,u))
									A=A[0],A[1],B[2]
									d-=adv
								B = segm[1][0],segm[1][1],surface
								tr_block.extend(self.helical(A,B,helicalRadius,phi,u))

						if segm[0][2] >hardcrust:# and segm[1][2]<=surface:
							if hardcrust< surface:
								if segm[1][2]<=hardcrust:	
									tr_block.append("(Helical in hard crust)" )
									A=A[0],A[1],surface
									d=A[2]-hardcrust
									adv=downPecking * hardcrustfeed
									while d > adv:
										B = segm[1][0],segm[1][1],max(segm[1][2],A[2]-adv)
										tr_block.extend(self.helical(A,B,helicalRadius,phi,u))
										A=A[0],A[1],B[2]
										d-=adv
									B = segm[1][0],segm[1][1],hardcrust
									tr_block.extend(self.helical(A,B,helicalRadius,phi,u))
						
						tr_block.append("(Helical to target )" )
						A=A[0],A[1],hardcrust
						d=A[2]-segm[1][2]
						adv=downPecking
						while d > adv:
							B = segm[1][0],segm[1][1],A[2]-adv
							tr_block.extend(self.helical(A,B,helicalRadius,phi,u))
							A=A[0],A[1],B[2]
							d-=adv
						B = segm[1][0],segm[1][1],segm[1][2]
						tr_block.extend(self.helical(A,B,helicalRadius,phi,u))
						tr_block.append("(Flatten)")
						tr_block.extend(self.helical(B,B,helicalRadius,phi,u))
						if round(helicalRadius,4)!=round(radius,4):
							tr_block.append("(Spiral adjustement)")
	#								tr_block.extend(self.trochoid(t_splice,B,B,radius,helicalRadius,phi,phi+4*pi,cw))
	#						steps=max(1,int(steps*radius*(spiral_twists)/2.0))
	#						steps=min(steps, 12*spiral_twists)
	#						steps*=spiral_twists
#							tr_block.append("(Spiral steps "+str(steps)+" in "+str(int((spiral_twists/2.)+1))+" twists)")
#							tr_block.append("(Spiral "+str(int((spiral_twists/2.)+1))+" twists)")
							tr_block.append("(Spiral "+str(spiral_twists)+" twists)")
							tr_block.extend(self.splice_generator(B,B,helicalRadius,radius,phi,phi-spiral_twists*2*pi, radians(-90),radians(-90),u,1.2*steps))
							tr_block.append("(Target diameter)")
#							tr_block.extend(self.helical(B,B,radius,phi,u))
							tr_block.extend(self.trochoid(t_splice,B,B,radius,radius,phi,phi,cw))

#						ascent
					elif segm[1][2] > segm[0][2]:
						tr_block.append("(Helical rapid ascentt "+"helicalRadius "+str(helicalRadius)+" )" )
						B = segm[1][0],segm[1][1],segm[1][2]
						tr_block.extend(self.helical(A,B,helicalRadius,phi,u))
	#					tr_block.append(CNC.grapid(center[0],center[1],center[2]))
	#					tr_block.extend(CNC.grapid(center))
	#					end of segment
	#					tr_block.append(CNC.gline(segm[1][0],segm[1][1],segm[1][2]))
#					oldsegm=segm
		tr_block.append("(-----------------------------------------)")
		tr_block.append(CNC.zsafe()) 			#<<< Move rapid Z axis to the safe height in Stock Material
		blocks.append(tr_block)
		self.finish_blocks(app, blocks)
	#----------------------------------------------------------
	#Convert polar to cartesian and add that to existing vector
	def pol2car(self, r, phi, a=[0,0]):
		return [round(a[0]+r*cos(phi),5),round(a[1]+r*sin(phi),5)]

	#Generate single trochoidal element between two points
	def trochoid(self, typesplice, A, B, oldradius, radius, oldphi,phi, cw=True):

		if self["splicesteps"] =="" or self["splicesteps"]<4:
			steps=4/(2*pi)
		else:
			steps=self["splicesteps"]/(2*pi)

		t_splice = typesplice
		block = []

		if cw:
			u = 1
			arc = "G2"
			cut_splice="G3"
		else:
			u = -1
			arc = "G3"
			cut_splice="G2"

#		phi = atan2(B[1]-A[1], B[0]-A[0])
#		step = sqrt((A[0]-B[0])**2+(A[1]-B[1])**2)

		l = self.pol2car(radius, phi+radians(90*u))
		r = self.pol2car(radius, phi+radians(-90*u))
		al = self.pol2car(oldradius, phi+radians(90*u), A)
		ar = self.pol2car(oldradius, phi+radians(-90*u), A)
		bl = self.pol2car(radius, phi+radians(90*u), B)
		br = self.pol2car(radius, phi+radians(-90*u), B)

#		prev_al = self.pol2car(oldradius, phi+radians(90*u), A)
#		prev_ar = self.pol2car(oldradius, phi+radians(-90*u), A)
#		old_l = self.pol2car(oldradius, oldphi+radians(90*u))
#		old_r = self.pol2car(oldradius, oldphi+radians(-90*u))

#		infinite radius
		inf_radius=1
#		inf_l = self.pol2car(500*radius, phi+radians(90*u))
#		inf_r = self.pol2car(500*radius, phi+radians(-90*u))
#		inf_Cl = self.pol2car(inf_radius*radius, phi+radians(-90*u), B)
#		inf_Cl= inf_Cl[0],inf_Cl[1],B[2]
#		inf_Cr = self.pol2car(inf_radius*radius, phi+radians(90*u), B)
#		inf_Cl= inf_Cr[0],inf_Cr[1],B[2]

		splice_dist=sqrt((br[0]-al[0])**2+(br[1]-al[1])**2)
		splice_radius=splice_dist/2.0



		# This schematic drawing represents naming convention
		# of points and vectors calculated in previous block
		#
		#    <--L---
		#          ---R-->
		#
		#        *   *
		#     *         *
		#    *           *
		#   BL     B     BR
		#    *           *
		#    *     ^     *
		#    *     |     *
		#    *     |     *
		#    *           *
		#   AL     A     AR
		#    *           *
		#     *         *
		#        *   *

		#TODO: improve strategies
#			block.append("g1 x"+str(al[0])+" y"+str(al[1])+" z"+str(A[2]))
#		First splice to next trochoid ========================================================
#		steps=self["splicesteps"]
#		steps = max(steps*oldradius,steps*radius)#self["splicesteps"]
#		steps = int(min(10,steps))#self["splicesteps"]
		
		block.append("(---------trochoid center x "+str(round(B[0],4))+" y "+str(round(B[1],4))+" ---------)")
		if t_splice == "came_back":
#			block.append(CNC.gline(round(A[0],5),round(A[1],5),round(B[2],5)))
			block.append(CNC.gline(round(B[0],5),round(B[1],5),round(B[2],5)))
#			block.append(arc+" x"+str(round(br[0],4))+" y"+str(round(br[1],4))+" R"+str(1.001*radius/2.0)+" z"+str(round(B[2],4))) 
			block.append(arc+" x"+str(round(br[0],4))+" y"+str(round(br[1],4))+" R"+str(self.roundup(radius/2.0,3))+" z"+str(round(B[2],4))) 
#			block.extend(self.splice_generator(steps,A,B,oldradius,radius/2.0,oldphi,phi,radians(270),radians(60),u))
#			block.append("(ppp)")
#			block.extend(self.splice_generator(steps,B,B,radius/2.0,radius,phi,phi,radians(60),radians(-90),u))

		else:
			block.append("( phi "+str(round(degrees(phi),2))+" oldphi "+str(round(degrees(oldphi),2))+" )")
			if oldphi!= phi:
				block.append("(=============== Direction changed olldhi ==================)")
			#if oldphi!=phi and t_splice!="Splices" and t_splice!="Warpedarc" and t_splice!="Circular one side rectified":
#				steps = int(15*radius)
			#	block.append("()")
			#	if t_splice!="Circular both sides rectified" :
				block.append("( Splice arch for direction change )")
	#				block.extend(self.splice_generator(A,A,oldradius,oldradius,oldphi,phi,radians(270),radians(-90),u))
	#			block.extend(self.curve_splice_generator(A,A,oldradius,oldradius,oldphi,phi,radians(270),radians(-90),u,steps))
				block.extend(self.curve_splice_generator(A,A,oldradius,oldradius,oldphi,phi,radians(270),radians(-90),u,4)) #<< steps=4
				block.append("(=============== End Direction changed ==================)")
	#				block.append("(new phi)")
	#				block.append(CNC.gline(old_ar[0],old_ar[1]))
	#				block.append(arc +" x"+str(ar[0])+" y"+str(ar[1])+" i"+str(-old_r[0])+" j"+str(-old_r[1])) #<<in adaptativee presents problems of location of the center in change of angle
	#				block.append(arc +" x"+str(prev_ar[0])+" y"+str(prev_ar[1])+" i"+str(-old_r[0])+" j"+str(-old_r[1])) #<<in adaptativee presents problems of location of the center in change of angle
	#				block.append(arc +" x"+str(prev_al[0])+" y"+str(prev_al[1])+" R"+str(oldradius))
	#				block.append(arc+" x"+str(prev_al[0])+" y"+str(prev_al[1])+" i"+str(-old_l[0])+" j"+str(-old_l[1])+" )")
	#				block.append(arc +" x"+str(prev_ar[0])+" y"+str(prev_ar[1])+" R"+str(oldradius))
			#	else:
			#		block.append("( Splice arch for direction change )")
	#				block.append(arc +" x"+str(al[0])+" y"+str(al[1])+" i"+str(old_r[0])+" j"+str(old_r[1]))
			#		block.append(arc +" x"+str(round(ar[0],4))+" y"+str(round(ar[1],4))+" R"+str(self.roundup(oldradius,3)))
			#		block.append(arc +" x"+str(round(al[0],4))+" y"+str(round(al[1],4))+" R"+str(self.roundup(oldradius,3)))
			#	block.append("()")

			if t_splice == "Splices":
#				block.append("(Soft steps "+str(steps)+" )")
				block.extend(self.splice_generator(A,B,oldradius,radius,oldphi,phi,radians(-90),radians(90),u,steps))

			elif t_splice == "Circular one side rectified":
	#			block.append(arc+" x"+str(al[0])+" y"+str(al[1])+" i"+str(old_l[0])+" j"+str(old_l[1])+" z"+str(round(B[2],5)))#<<in adaptativee presents problems of location of the center in change of angle
				block.append(arc+" x"+str(round(al[0],4))+" y"+str(round(al[1],4))+" R"+str(self.roundup(oldradius,3))+" z"+str(round(B[2],5)))
				block.append("g1 x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" z"+str(round(B[2],4)))

			elif t_splice == "Circular both sides rectified" :
				block.append("g1 x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" z"+str(round(B[2],4)))

			elif t_splice == "Straight":
				block.append("g1 x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" z"+str(round(B[2],4)))

			elif t_splice == "Warpedarc":
				warped_radius=(sqrt((ar[0]-bl[0])**2+(ar[1]-bl[1])**2))/2.0
				block.append("(Warpedarc)")
				block.append("(control previous position arx "+str(round(ar[0],4))+" ary "+str(round(ar[1],4))+" R "+ str(warped_radius)+" )")
				block.append(arc+" x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" R"+str(self.roundup(warped_radius,3))+" z"+str(round(B[2],5)))

			elif t_splice == "Straight on side rectified":			
				block.append("g1 x"+str(round(al[0],4))+" y"+str(round(al[1],4)))
				block.append("g1 x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" z"+str(round(B[2],4)))

			elif t_splice == "Cut":
				block.append("g1 x"+str(round(ar[0],4))+" y"+str(round(ar[1],4)))
				block.append(cut_splice+"x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+"R"+str(self.roundup(splice_radius,3))+" z"+str(round(B[2],4)))

	#		========================================================================================================
		#	cut
#			block.append(arc+" x"+str(br[0])+" y"+str(br[1])+" i"+str(r[0])+" j"+str(r[1])+" z"+str(round(B[2],5)))
			block.append("(cuting)") 
			block.append(arc+" x"+str(round(br[0],4))+" y"+str(round(br[1],4))+" R"+str(self.roundup(radius,3))+" z"+str(round(B[2],4))) 
		#	block.append("(cut)")
		#	========================================================================================================
			if t_splice== "Circular both sides rectified":
				block.append("g1 x"+str(round(ar[0],4))+" y"+str(round(ar[1],4))+" z"+str(round(A[2],4)))
	#			block.append(arc+" x"+str(al[0])+" y"+str(al[1])+" i"+str(old_l[0])+" j"+str(old_l[1])+" z"+str(round(A[2],5)))
				block.append(arc+" x"+str(round(al[0],4))+" y"+str(round(al[1],4))+" R"+str(self.roundup(oldradius,3))+" z"+str(round(A[2],4))) #<<in adaptativee presents problems of location of the center in change of angle
				block.append("g1 x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" z"+str(round(B[2],4)))

		return block
	#--------------------------------------------------------------
	def splice_generator(self, C1, C2, r1, r2, phi1,phi2,alpha_1,alpha_2, u,steps):
		block = []
#		steps=self["splicesteps"]/(2*pi)
#			when "Soft Arc" splice from AR to BL		
#			C1=A  # center1
#			C2=B  # center2
#			r1=radius
#			r2=radius
#		if phi1>=2*pi:
#			phi1-=2*pi
#		elif phi1<=-2*pi:
#			phi1+02*pi
#		if phi2-phi1==3*pi/2:
#			steps*=1.5
#			if u==1:
#				alpha_1+=2*pi

		alpha1 =phi1+alpha_1*u
		alpha2=phi2+alpha_2*u
#		block.append("(original alpha1 "+str(round(degrees(alpha1),2))+" original alpha2 "+str(round((alpha2),2))+" )")
#		secure direction of rotation splice		
#		if C1[0]!=C2[0] or C1[1]!=C2[2]:
#		block.append("(ppp)")
		if u==1:
			if alpha1<=alpha2:
				alpha1+=2*pi
#				block.append("(+360)")
				if alpha1<=alpha2:
					alpha1+=2*pi
#					block.append("(+360)")
#						steps*=2
		elif u==-1:		
			if alpha1>=alpha2:
				alpha1-=2*pi
				if alpha1>=alpha2:
					alpha1-=2*pi
#						steps*=2
		steps=int(abs(steps*(alpha2-alpha1))) 
		if steps==0: steps=1
#			delta or increas in values radius, anle, center position
		d_r=r2-r1
		d_r=d_r/steps

		d_a=alpha2- alpha1
		d_a=d_a/steps

		d_c_x=(C2[0]-C1[0])/steps
		d_c_y=(C2[1]-C1[1])/steps
		d_c_z=(C2[2]-C1[2])/steps
		r=r1
		alpha=alpha1
		C=C1
		i=0

		a0 = self.pol2car(r1,alpha1,C1)
#		block.append("(phi "+str(round(degrees(alpha),4)+90.0)+" first splice point )")
		block.append("(alpha1 "+str(round(degrees(alpha1),2))+" alpha2 "+str(round(degrees(alpha2),2))+" first splice point )")
		block.append("(phi-oldphi "+str(round(degrees((phi2-phi1)),2))+" steps "+str(steps)+" )")
#		block.append("(R1 "+ str(r1)+" R2 "+str(r2)+" )")
#		block.append("g1 x"+str(C1[0])+" y"+str(C[1])+" z"+str(round(C[1],5)))
		block.append("g1 x"+str(round(a0[0],4))+" y"+str(round(a0[1],4)))#+" z"+str(round(C[1],5)))

		while i<(steps):#-0.1*steps): #<<<<solution not very elegat by mistake last splice with many steps (accumulation of error?)
			r+=d_r
			alpha+=d_a
			C=C[0]+d_c_x, C[1]+d_c_y,C[2]+d_c_z
#			ai = self.pol2car(radius, phi+radians(-120*u),A)
			ai = self.pol2car(r, alpha,C)
#				block.append(("g0 x ")+str(round(C1[0],2))+ " y"+str(round(C1[1],2))+" z"+str(round(C1[2],2)))
#				block.append("(phi "+str(round( (phi+radians(-90*u))*57.29,2))+" )")
#				block.append("(phi "+str(round(phi*57.29,2))+" alpha "+str(round(alpha1*57.29,2))+" )")
#			block.append("(Ax "+ str(C1[0])+" Ay "+str(C1[1])+ " Bx "+str(C2[0])+" By "+str(C1[2])+" )")
#			block.append("(g1 0"+str(ai[0])+" y"+str(ai[1])+" z"+str(round(C[2],5))+" )")
			block.append("(alpha "+str(round(degrees(alpha),2))+" )")
			block.append("g1 x"+str(round(ai[0],4))+" y"+str(round(ai[1],4))+" z"+str(round(C[2],4)))
			i+=1
		return block		
	#--------------------------------------------------------------
	def curve_splice_generator(self, C1, C2, r1, r2, phi1,phi2,alpha_1,alpha_2, u,steps):
		block = []
#		if u == 1:
#			arc = "G2"
#		else:
#			arc = "G3"

#		steps=self["splicesteps"]/(2*pi)
#			when "Soft Arc" splice from AR to BL		
#			C1=A  # center1
#			C2=B  # center2
#			r1=radius
#			r2=radius
#		if phi1>=2*pi:
#			phi1-=2*pi
#		elif phi1<=-2*pi:
#			phi1+02*pi
#		if phi2-phi1==3*pi/2:
#			steps*=1.5
#			if u==1:
#				alpha_1+=2*pi

		alpha1 =phi1+alpha_1*u
		alpha2=phi2+alpha_2*u
#		block.append("(original alpha1 "+str(round(degrees(alpha1),2))+" original alpha2 "+str(round((alpha2),2))+" )")
#		secure direction of rotation splice		
#		if C1[0]!=C2[0] or C1[1]!=C2[2]:
#		block.append("(ppp)")
		if u==1:
			arc = "G2"
			if alpha1<=alpha2:
				alpha1+=2*pi
#				block.append("(+360)")
				if alpha1<=alpha2:
					alpha1+=2*pi
#					block.append("(+360)")
#						steps*=2
		elif u==-1:		
			arc = "G3"
			if alpha1>=alpha2:
				alpha1-=2*pi
				if alpha1>=alpha2:
					alpha1-=2*pi
#						steps*=2
#		steps=int(abs(steps*(alpha2-alpha1))) 
		if steps==0: steps=2
#			delta or increas in values radius, anle, center position
		d_r=r2-r1
		d_r=d_r/steps

		d_a=alpha2- alpha1
		d_a=d_a/steps

		d_c_x=(C2[0]-C1[0])/steps
		d_c_y=(C2[1]-C1[1])/steps
		d_c_z=(C2[2]-C1[2])/steps
		r=r1
		alpha=alpha1
		C=C1
		i=0

		a0 = self.pol2car(r1,alpha1,C1)
#		block.append("(phi "+str(round(degrees(alpha),4)+90.0)+" first splice point )")
		block.append("(alpha1 "+str(round(degrees(alpha1),2))+" alpha2 "+str(round(degrees(alpha2),2))+" first splice point )")
		block.append("(phi-oldphi "+str(round(degrees((phi2-phi1)),2))+" steps "+str(steps)+" )")
#		block.append("(R1 "+ str(r1)+" R2 "+str(r2)+" )")
#		block.append("g1 x"+str(C1[0])+" y"+str(C[1])+" z"+str(round(C[1],5)))
		block.append("g1 x"+str(round(a0[0],4))+" y"+str(round(a0[1],4)))#+" z"+str(round(C[1],5)))

		while i<(steps):#-0.1*steps): #<<<<solution not very elegat by mistake last splice with many steps (accumulation of error?)
			r+=d_r
			alpha+=d_a
			C=C[0]+d_c_x, C[1]+d_c_y,C[2]+d_c_z
#			ai = self.pol2car(radius, phi+radians(-120*u),A)
			ai = self.pol2car(r, alpha,C)
#				block.append(("g0 x ")+str(round(C1[0],2))+ " y"+str(round(C1[1],2))+" z"+str(round(C1[2],2)))
#				block.append("(phi "+str(round( (phi+radians(-90*u))*57.29,2))+" )")
#				block.append("(phi "+str(round(phi*57.29,2))+" alpha "+str(round(alpha1*57.29,2))+" )")
#			block.append("(Ax "+ str(C1[0])+" Ay "+str(C1[1])+ " Bx "+str(C2[0])+" By "+str(C1[2])+" )")
#			block.append("(g1 0"+str(ai[0])+" y"+str(ai[1])+" z"+str(round(C[2],5))+" )")
			block.append("(alpha "+str(round(degrees(alpha),2))+" )")
#			block.append("g1 x"+str(round(ai[0],4))+" y"+str(round(ai[1],4))+" z"+str(round(C[2],4)))
			block.append(arc+" x"+str(round(ai[0],4))+" y"+str(round(ai[1],4))+" R "+str(self.roundup(r,3))+" z"+str(round(C[2],4)))
			i+=1
		return block		
	#--------------------------------------------------------------
	def helical(self, A, C, radius, phi, u): #<< A[2], C=B
		block = []
		t_splice = self["TypeSplice"]
#		if cw:
		if	u == 1:
			arc = "G2"
		else:
			u = -1
			arc = "G3"

#		alphal=phi+radians(-90*u)
		l = self.pol2car(radius, phi+radians(90*u))
		r = self.pol2car(radius, phi+radians(-90*u))
		bl = self.pol2car(radius, phi+radians(90*u), C)
		br = self.pol2car(radius, phi+radians(-90*u), C)

		radius=round(radius,4)
		l[0]=round(l[0],4)
		l[1]=round(l[1],4)
		r[0]=round(r[0],4)
		r[1]=round(r[1],4)

		# RESOLVE CONFLICT G2 G3 I J 
		obteinedradius= sqrt(l[0]**2+l[1]**2)
#		block.append("(obteinedradius "+str(obteinedradius)+" )")
		if obteinedradius>radius:
			block.append("(error G2 G3: radius "+str(radius)+" radius with sqrt "+str(obteinedradius)+" )") 
#			radius+=0.0001
			radius=self.roundup(obteinedradius,3)
			block.append("(new radius "+ str(radius)+" )")
		if t_splice=="Circular both sides rectified":
#			block.append(arc+" x"+str(Cl[0])+" y"+str(Cl[1])+" i"+str(l[0])+" j"+str(l[1]))						
			block.append(arc+" x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" i"+str(-l[0])+" j"+str(-l[1])+ " z" +str(round(C[2],4)))
		else:
#			block.append(arc+" x"+str(Cl[0])+" y"+str(Cl[1])+" i"+str(l[0])+" j"+str(l[1])+ " z" +str(C[2]))
			block.append("G1 x"+str(round(br[0],4))+" y"+str(round(br[1],4))) 
#			block.append(arc+" x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" i"+str(r[0])+" j"+str(-r[1])+" z"+str(round(C[2],4))) 

#			block.append(arc+" x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" i"+str(-r[0])+" j"+str(-r[1])+" z"+str(round(C[2],4))) 
#			block.append(arc+" x"+str(round(br[0],4))+" y"+str(round(br[1],4))+" i"+str(r[0])+" j"+str(r[1])+" z"+str(round(C[2],4))) 
	#		block.append(arc+" x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" R"+str(radius)+" z"+str(round(C[2],4))) 
	#		block.append(arc+" x"+str(round(br[0],4))+" y"+str(round(br[1],4))+" R"+str(radius)+" z"+str(round(C[2],4))) 
			z_interm=(A[2]+C[2])/2.0
			block.append(arc+" x"+str(round(bl[0],4))+" y"+str(round(bl[1],4))+" R"+str(self.roundup(radius,4))+" z"+str(round(z_interm,4))) 
			block.append(arc+" x"+str(round(br[0],4))+" y"+str(round(br[1],4))+" R"+str(self.roundup(radius,4))+" z"+str(round(C[2],4))) 

		return block		
	#--------------------------------------------------------------
	def roundup(self, number, decimals):
	#	decimals=4
		if number<0:
			sign=-1
		else:
			sign=1
	#	if abs(number-round(number,decimals))>0:
	#		number=round(number,decimals)+0.0001*sign
	#	else:
	#		number=round(number,decimals)
	#		number=round(number,decimals)+0.0001*sign
	#		number=round(number,decimals)+(1/(10**decimals))*sign
	#	number=round(number,decimals)+0.0001*sign#/(10**decimals)*sign
	#	number=round(number,decimals)+((1/10)**decimals)*sign
		number=round(number,decimals)+0.1**decimals*sign
		return number


	#Insert created blocks
	def finish_blocks(self, app, blocks):
		active = app.activeBlock()
#		if active==0: active=1
#		active=2
		app.gcode.insBlocks(active+1, blocks, "Trochs")
		app.refresh()
		app.setStatus(_("Trochoid Generated"))



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
import sys


IS_PYTHON2 = (sys.version_info[0] == 2)


def b(s, encoding='ascii', errors='replace'):  # pragma: no cover
    if IS_PYTHON2:
        return bytes(s)
    else:
        if isinstance(s, str):
            return bytes(s, encoding, errors)
        else:
            return s
        # return bytes(s, encoding, errors)


def s(s):  # pragma: no cover
    if IS_PYTHON2:
        return bytes(s)
    else:
        return s



# -*- coding: ascii -*-
# $Id$
#
# Author: vvlachoudis@gmail.com
# Date: 24-Aug-2014

from __future__ import absolute_import
from __future__ import print_function
try:
	import cv2 as cv
except ImportError:
	cv = None

try:
	import numpy as np
except ImportError:
	np = None

try:
	from PIL import Image, ImageTk
except ImportError:
	cv = None

import Utils


#-------------------------------------------------------------------------------
def hasOpenCV(): return cv is not None


#===============================================================================
# Camera processing class
# A wrapper to opencv needed functions
#===============================================================================
class Camera:
	#-----------------------------------------------------------------------
	# prefix is the prefix to get configuration parameters from ini
	#-----------------------------------------------------------------------
	def __init__(self, prefix=""):
		if cv is None: return
		self.prefix  = prefix
		self.idx     = Utils.getInt("Camera", prefix)
		self.props   = self._getCameraProperties(prefix)
		self.camera  = None
		self.image   = None
		self.frozen  = None
		self.imagetk = None

	def _getCameraProperties(self, prefix):
		""" Gather user-defined camera configuration properties

		Allows user to specify parameters like::

		    [Camera]
		    aligncam_height = 640
		    aligncam_width = 480

		to adjust the capture settings of the 'aligncam' camera.

		:param str prefix: Prefix to gather
		:returns: Dictionary mapping property suffix with a 2-tuple of
			a function defining the data type, and a CV_CAP_PROP
			property.
		"""
		try:
			POSSIBLE_PROPERTIES = {
				'height'     : (Utils.getInt, cv.CAP_PROP_FRAME_HEIGHT),
				'width'      : (Utils.getInt, cv.CAP_PROP_FRAME_WIDTH),
				'fps'        : (Utils.getInt, cv.CAP_PROP_FPS),
				'codec'      : (Utils.getStr, cv.CAP_PROP_FOURCC),
				'brightness' : (Utils.getInt, cv.CAP_PROP_BRIGHTNESS),
				'contrast'   : (Utils.getInt, cv.CAP_PROP_CONTRAST),
				'saturation' : (Utils.getInt, cv.CAP_PROP_SATURATION),
				'hue'        : (Utils.getInt, cv.CAP_PROP_HUE),
				'gain'       : (Utils.getInt, cv.CAP_PROP_GAIN),
				'exposure'   : (Utils.getInt, cv.CAP_PROP_EXPOSURE),
			}
		except AttributeError:
			POSSIBLE_PROPERTIES = {
				'height'     : (Utils.getInt, cv.cv.CV_CAP_PROP_FRAME_HEIGHT,),
				'width'      : (Utils.getInt, cv.cv.CV_CAP_PROP_FRAME_WIDTH,),
				'fps'        : (Utils.getInt, cv.cv.CV_CAP_PROP_FPS,),
				'codec'      : (Utils.getStr, cv.cv.CV_CAP_PROP_FOURCC,),
				'brightness' : (Utils.getInt, cv.cv.CV_CAP_PROP_BRIGHTNESS,),
				'contrast'   : (Utils.getInt, cv.cv.CV_CAP_PROP_CONTRAST,),
				'saturation' : (Utils.getInt, cv.cv.CV_CAP_PROP_SATURATION,),
				'hue'        : (Utils.getInt, cv.cv.CV_CAP_PROP_HUE,),
				'gain'       : (Utils.getInt, cv.cv.CV_CAP_PROP_GAIN,),
				'exposure'   : (Utils.getInt, cv.cv.CV_CAP_PROP_EXPOSURE,),
			}

		UNSPECIFIED = object()
		properties_set = {}

		for key, data in POSSIBLE_PROPERTIES.items():
			fn, prop = data
			value = fn(
				'Camera',
				'_'.join([prefix, key]),
				default=UNSPECIFIED
			)

			if value is not UNSPECIFIED:
				properties_set[prop] = value

		return properties_set

	#-----------------------------------------------------------------------
	def isOn(self):
		if cv is None: return False
		return self.camera is not None and self.camera.isOpened()

	#-----------------------------------------------------------------------
	def start(self):
		if cv is None: return
		self.camera = cv.VideoCapture(self.idx)

		for prop_id, prop_value in self.props.items():
			self.camera.set(prop_id, prop_value)

		if self.camera is None: return
		s, self.image = self.camera.read()
		if not self.camera.isOpened():
			self.stop()
			return False
		self.set()
		return True

	#-----------------------------------------------------------------------
	def stop(self):
		if cv is None or self.camera is None: return
		self.camera.release()
#		del self.camera
		self.camera = None

	#-----------------------------------------------------------------------
	def set(self):
		width = Utils.getInt("Camera", self.prefix+"_width",  0)
		if width: self.camera.set(3, width)
		height = Utils.getInt("Camera", self.prefix+"_height",  0)
		if height: self.camera.set(4, height)
		self.angle = Utils.getInt("Camera", self.prefix+"_angle")//90 % 4
		self.rotation = Utils.getFloat("Camera", self.prefix+"_rotation")
		self.xcenter = Utils.getFloat("Camera", self.prefix+"_xcenter")
		self.ycenter = Utils.getFloat("Camera", self.prefix+"_ycenter")
#		self.camera.set(38, 3) # CV_CAP_PROP_BUFFERSIZE

	#-----------------------------------------------------------------------
	# Read one image and rotated if needed
	#-----------------------------------------------------------------------
	def read(self):
		s,self.image = self.camera.read()
		if s:
			self.image = self.rotate90(self.image)
		else:
			self.stop()
		self.original = self.image

		if self.frozen is not None:
			self.image = cv.addWeighted(self.image, 0.7, self.frozen, 0.3, 0.0)
		return s

	#-----------------------------------------------------------------------
	# Save image to file
	#-----------------------------------------------------------------------
	def save(self, filename):
		cv.imwrite(filename, self.original)

	#-----------------------------------------------------------------------
	# Rotate image in steps of 90deg
	#-----------------------------------------------------------------------
	def rotate90(self, image):
		if self.rotation > 0:
			rows, cols = image.shape[:2]
			m = cv.getRotationMatrix2D((cols/2, rows/2), self.rotation,1)
			image = cv.warpAffine(image, m, (cols,rows),
					None,
					cv.INTER_LINEAR, cv.BORDER_CONSTANT,
					(255, 255, 255))
			rows, cols = image.shape[:2]
			t = np.float32([
				[1, 0, -self.xcenter],
				[0, 1, -self.ycenter]])
			image = cv.warpAffine(image,t,(cols,rows), None,
					cv.INTER_LINEAR, cv.BORDER_CONSTANT,
					(255, 255, 255))
			return image
		if self.angle == 1:	# 90 deg
			return cv.transpose(cv.flip(image,1))
		elif self.angle == 2: # 180 deg
			return cv.flip(image,-1)
		elif self.angle == 3: # 270 deg
			return cv.flip(cv.transpose(image),1)
		else:
			return image

	#-----------------------------------------------------------------------
	# Resize image up to a maximum width,height
	#-----------------------------------------------------------------------
	def resize(self, factor, maxwidth, maxheight):
		if factor==1.0: return
		h,w = self.image.shape[:2]
		wn = int(w*factor)
		hn = int(h*factor)
		if wn>maxwidth or hn>maxheight:
			# crop the image to match max
			wn = int(maxwidth/factor)//2
			hn = int(maxheight/factor)//2
			w2 = w//2
			h2 = h//2
			left  = max(w2-wn,0)
			right = min(w2+wn,w-1)
			top   = max(h2-hn,0)
			bottom = min(h2+hn,h-1)
			self.image = self.image[top:bottom,left:right]
		try:
			self.image = cv.resize(self.image, (0,0), fx=factor, fy=factor)
		except:
			# FIXME Too much zoom out, results in void image!
			#self.image = None
			pass

	#-----------------------------------------------------------------------
	# Canny edge detection
	#-----------------------------------------------------------------------
	def canny(self, threshold1, threshold2):
		edge = cv.cvtColor(cv.Canny(self.image, threshold1, threshold2), cv.COLOR_GRAY2BGR)
		self.image = cv.addWeighted(self.image, 0.9, edge, 0.5, 0.0)

	#-----------------------------------------------------------------------
	# Freeze and overlay image
	#-----------------------------------------------------------------------
	def freeze(self, f):
		if f:
			self.frozen = self.image.copy()
		else:
			self.frozen = None

	#-----------------------------------------------------------------------
	# Return center portion of the image to be used as a template
	#-----------------------------------------------------------------------
	def getCenterTemplate(self, r):
		h,w = self.original.shape[:-1]
		w2 = w//2
		h2 = h//2
		return self.original[h2-r:h2+r, w2-r:w2+r]

	#-----------------------------------------------------------------------
	# return location of matching template
	#-----------------------------------------------------------------------
	def matchTemplate(self, template):
		method = cv.TM_CCOEFF_NORMED
		res = cv.matchTemplate(self.original, template, method)
		min_val, max_val, min_loc, max_loc = cv.minMaxLoc(res)

		h,w = self.original.shape[:-1]
		w2 = w//2
		h2 = h//2

		h,w = template.shape[:-1]
		r = w//2

		# If the method is TM_SQDIFF or TM_SQDIFF_NORMED, take minimum
		if method in (cv.TM_SQDIFF, cv.TM_SQDIFF_NORMED):
			top_left = min_loc
		else:
			top_left = max_loc
		#bottom_right = (top_left[0]+2*r, top_left[1]+2*r)
		dx= w2-r - top_left[0]
		dy= h2-r - top_left[1]
		#cv.rectangle(img, top_left, bottom_right, 255, 2)
		#print "Match=",dx,dy
		return dx,dy

	#-----------------------------------------------------------------------
	# Convert to Tk image
	#-----------------------------------------------------------------------
	def toTk(self):
		if self.image is None: return None
		self.imagetk = ImageTk.PhotoImage(
					image=Image.fromarray(
						cv.cvtColor(self.image, cv.COLOR_BGR2RGB), "RGB"))
		return self.imagetk



#!/usr/bin/python
# -*- coding: ascii -*-

# Author: @harvie Tomas Mudrunka
# Date: 7 july 2018

from __future__ import absolute_import
from __future__ import print_function
__author__ = "@harvie Tomas Mudrunka"
#__email__  = ""

__name__ = _("Trochoidal")
__version__ = "0.0.2"

import math
import os.path
import re
from CNC import CNC,Block
from ToolsPage import Plugin
from math import pi, sqrt, sin, cos, asin, acos, atan2, hypot, degrees, radians, copysign, fmod


class Tool(Plugin):
	__doc__ = _("""Trochoidal g-code postprocessor""")			#<<< This comment will be show as tooltip for the ribbon button

	def __init__(self, master):
		Plugin.__init__(self, master,"Trochoidal")
		#Helical_Descent: is the name of the plugin show in the tool ribbon button
		self.icon = "trochoidal"			#<<< This is the name of file used as icon for the ribbon button. It will be search in the "icons" subfolder
		self.group = "CAM"	#<<< This is the name of group that plugin belongs
		#Here we are creating the widgets presented to the user inside the plugin
		#Name, Type , Default value, Description
		self.variables = [			#<<< Define a list of components for the GUI
			("name"    ,    "db" ,    "", _("Name")),							#used to store plugin settings in the internal database
			("cw"    ,    "bool" ,    True, _("Clockwise")),
			("circ"    ,    "bool" ,    False, _("Circular")),
			("evenspacing"    ,    "bool" ,    True, _("Even spacing across segment")),
			("entry"    ,    "bool" ,    False, _("Trochoid entry (prepare for helicut)")),
			("rdoc"    ,    "mm" ,    "0.2", _("Radial depth of cut (<= cutter D * 0.4)")),
			("dia"    ,    "mm" ,    "3", _("Trochoid diameter (<= cutter D)")),
			("feed"    ,    "mm" ,    "2000", _("Feedrate"))
		]
		self.buttons.append("exe")  #<<< This is the button added at bottom to call the execute method below


	# ----------------------------------------------------------------------
	# This method is executed when user presses the plugin execute button
	# ----------------------------------------------------------------------
	def execute(self, app):
		feed = self["feed"]
		rdoc = self["rdoc"]
		radius = self["dia"]/2
		cw = self["cw"]
		circ = self["circ"]
		evenspacing = self["evenspacing"]

		if cw: cwtext = 'cw'
		else: cwtext = 'ccw'

		if cw: arcg = 'g2'
		else: arcg = 'g3'

		#print("go!")
		blocks  = []
		#Loop over selected blocks
		for bid in app.editor.getSelectedBlocks():
			#print(blocks[bid])
			path = app.gcode.toPath(bid)[0]
			#print(path)

			#create new block which encorporates trochoidal path
			block = Block("trochoid "+cwtext+" "+str(radius*2)+"+"+str(rdoc))
			block.append("F"+str(feed))

			entry = self["entry"]

			A=path[0].A
			block.append("g0 x"+str(A[0])+" y"+str(A[1]))
			block.append("G1 Z0")
			#Loop over segments within path
			for segment in path:
				#print(segment.A)
				#create Block for circular entry into path
				if entry:
					eblock = Block("trochoid-in")
					eblock.append("G0 Z0")
					eblock.append("G0 x"+str(segment.A[0])+" y"+str(segment.A[1]-radius))
					eblock.append("G2 x"+str(segment.A[0])+" y"+str(segment.A[1]-radius)+" i"+str(0)+" j"+str(radius))
					blocks.append(eblock)
					entry = False

				#Continuity BEGINING
				# calculate number of subsegments to be transformed to trochoidal motion
				srdoc = rdoc
				segmentLength = segment.length()
				subsegs = segmentLength//rdoc
				remainder = segmentLength%rdoc

				#Compensate for uneven spacing
				if evenspacing:
					if remainder != 0:
						subsegs = subsegs+1
						srdoc = segmentLength/subsegs
						remainder = 0

				#Loop over subsegments of segment
				startSegment=True
				for i in range(1,int(subsegs)+1):
					pos=i*srdoc

					B = segment.distPoint(pos)
					block.extend(self.trochoid(A,B,radius,cw,circ,startSegment))
					A = B
					# Lead in performed, so clear flag
					startSegment=False
				# Process remainder
				if remainder > 0:
					B = segment.distPoint(segmentLength)
					block.extend(self.trochoid(A,B,radius,cw,circ,startSegment))
					A = B					

				#Continuity END
				#Move bit to center of cut (B) at end of segment
				block.append(arcg+" x"+str(segment.B[0])+" y"+str(segment.B[1])+" r"+str(radius/2))

			blocks.append(block)


		active = app.activeBlock()
		app.gcode.insBlocks(active, blocks, "Trochoidal created") 	#<<< insert blocks over active block in the editor
		app.refresh()                                                   #<<< refresh editor
		app.setStatus(_("Generated: Trochoidal"))                       #<<< feed back result


	#Convert polar to cartesian and add that to existing vector
	def pol2car(self, r, phi, a=[0,0]):
		return [round(a[0]+r*cos(phi),4),round(a[1]+r*sin(phi),4)]

	#Generate single trochoidal element between two points
	def trochoid(self, A, B, radius, cw=True, circular=False, startSegment=False):
		block = []

		if cw:
			u = 1
			arc = "G2"
		else:
			u = -1
			arc = "G3"


		phi = atan2(B[1]-A[1], B[0]-A[0])
		step = sqrt((A[0]-B[0])**2+(A[1]-B[1])**2)

		l = self.pol2car(radius, phi+radians(90*u))
		r = self.pol2car(radius, phi+radians(-90*u))
		al = self.pol2car(radius, phi+radians(90*u), A)
		ar = self.pol2car(radius, phi+radians(-90*u), A)
		bl = self.pol2car(radius, phi+radians(90*u), B)
		br = self.pol2car(radius, phi+radians(-90*u), B)

		# This schematic drawing represents naming convention
		# of points and vectors calculated in previous block
		#
		#    <--L---
		#          ---R-->
		#
		#        *   *
		#     *         *
		#    *           *
		#   BL     B     BR
		#    *           *
		#    *     ^     *
		#    *     |     *
		#    *     |     *
		#    *           *
		#   AL     A     AR
		#    *           *
		#     *         *
		#        *   *

		#TODO: improve strategies
		# This is lead in circle of segment (moving from center (A) to cutting edge (AL))
		if startSegment:
			block.append(arc+" x"+str(al[0])+" y"+str(al[1])+" r"+str(radius/2))
		# This is circular cutting cycle (very simple, less motion cycles but not so accurate AL->BL->BR-BL)
		if circular:
			block.append("g1 x"+str(bl[0])+" y"+str(bl[1]))
			block.append(arc+" x"+str(br[0])+" y"+str(br[1])+" i"+str(r[0])+" j"+str(r[1]))
			block.append(arc+" x"+str(bl[0])+" y"+str(bl[1])+" i"+str(l[0])+" j"+str(l[1]))
		# This is more detailed, performing complete cycle AL->BL->BR->AR->AL->BL
		else:
			block.append("g1 x"+str(bl[0])+" y"+str(bl[1]))
			block.append(arc+" x"+str(br[0])+" y"+str(br[1])+" i"+str(r[0])+" j"+str(r[1]))
			block.append("g1 x"+str(ar[0])+" y"+str(ar[1]))
			block.append(arc+" x"+str(al[0])+" y"+str(al[1])+" i"+str(l[0])+" j"+str(l[1]))
			block.append("g1 x"+str(bl[0])+" y"+str(bl[1]))

		return block



#!/usr/bin/python
# -*- coding: ascii -*-

# Author: @harvie Tomas Mudrunka
# Date: 25 sept 2018

from __future__ import absolute_import
from __future__ import print_function
__author__ = "@harvie Tomas Mudrunka"
#__email__  = ""

__name__ = _("Linearize")
__version__ = "0.2"

import math
import os.path
import re
from CNC import CNC,Block
from bpath import eq, Path, Segment
from ToolsPage import Plugin
from math import pi, sqrt, sin, cos, asin, acos, atan2, hypot, degrees, radians, copysign, fmod


class Tool(Plugin):
	__doc__ = _("""G-Code linearizer""")			#<<< This comment will be show as tooltip for the ribbon button

	def __init__(self, master):
		Plugin.__init__(self, master,"Linearize")
		self.icon = "linearize"			#<<< This is the name of file used as icon for the ribbon button. It will be search in the "icons" subfolder
		self.group = "CAM"	#<<< This is the name of group that plugin belongs
		#self.oneshot = True
		#Here we are creating the widgets presented to the user inside the plugin
		#Name, Type , Default value, Description
		self.variables = [			#<<< Define a list of components for the GUI
			("name"    ,    "db" ,    "", _("Name")),							#used to store plugin settings in the internal database
			("maxseg", "mm", "1", _("segment size"), _("Maximal length of resulting lines, smaller number means more precise output and longer g-code. Length will be automaticaly truncated to be even across whole subdivided segment.")),
			("splitlines", "bool", False, _("subdiv lines"), _("Also subdivide the lines. Otherwise only arcs and splines will be subdivided"))
		]
		self.buttons.append("exe")  #<<< This is the button added at bottom to call the execute method below
		self.help = """
This plugin will subdivide the toolpath in such way, that all segments are broken to very short lines, which means that there will be no arcs or splines.
This is exact opposite of "arcfit" plugin. This is not very usefull for common CNC operation. However this might be useful when you need to proces arcs in CAD/CAM software which only support straight lines.
It usualy happens that new development features and plugins in bCNC only support straight lines and they add support for arcs later. So if you are early adopter of development features or encounter arc-related bug, you might try to use this plugin to convert your g-code to lines before working using these new features.
Also if you are working with some primitive motion controller, which only supports straight lines (G1). You might use this plugin to preprocess the g-code to get support for arcs (G2 and G3).

If you set the segment size large enough, you can even use this to create inscribed polygons in arcs and circles.
"""


	# ----------------------------------------------------------------------
	# This method is executed when user presses the plugin execute button
	# ----------------------------------------------------------------------
	def execute(self, app):
		maxseg = self.fromMm("maxseg")
		splitlines = self["splitlines"]

		#print("go!")
		blocks  = []
		for bid in app.editor.getSelectedBlocks():
			if len(app.gcode.toPath(bid)) < 1: continue

			#nblock = Block("flat "+app.gcode[bid].name())
			#for i in app.gcode[bid]:
			#	nblock.append(re.sub(r"\s?z-?[0-9\.]+","",i))
			#blocks.append(nblock)

			eblock = Block("lin "+app.gcode[bid].name())
			opath = app.gcode.toPath(bid)[0]
			npath = opath.linearize(maxseg, splitlines)
			eblock = app.gcode.fromPath(npath,eblock)
			blocks.append(eblock)



		#active = app.activeBlock()
		#if active == 0: active+=1
		active=-1 #add to end
		app.gcode.insBlocks(active, blocks, "Linearized") #<<< insert blocks over active block in the editor
		app.refresh()                                                                                           #<<< refresh editor
		app.setStatus(_("Generated: Linearize"))                           #<<< feed back result
		#app.gcode.blocks.append(block)



#!/bin/env python
#
# Copyright and User License
# ~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright Vasilis.Vlachoudis@cern.ch for the
# European Organization for Nuclear Research
#
# Please consult the documentation for the license
#
# DISCLAIMER
# ~~~~~~~~~~
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY, OF
# SATISFACTORY QUALITY, AND FITNESS FOR A PARTICULAR PURPOSE
# OR USE ARE DISCLAIMED. THE COPYRIGHT HOLDERS AND THE
# AUTHORS MAKE NO REPRESENTATION THAT THE SOFTWARE AND
# MODIFICATIONS THEREOF, WILL NOT INFRINGE ANY PATENT,
# COPYRIGHT, TRADE SECRET OR OTHER PROPRIETARY RIGHT.
#
# LIMITATION OF LIABILITY
# ~~~~~~~~~~~~~~~~~~~~~~~
# THE COPYRIGHT HOLDERS AND THE AUTHORS SHALL HAVE NO
# LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL,
# CONSEQUENTIAL, EXEMPLARY, OR PUNITIVE DAMAGES OF ANY
# CHARACTER INCLUDING, WITHOUT LIMITATION, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES, LOSS OF USE, DATA OR PROFITS,
# OR BUSINESS INTERRUPTION, HOWEVER CAUSED AND ON ANY THEORY
# OF CONTRACT, WARRANTY, TORT
# LIABILITY OR OTHERWISE, ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGES.
#
# Author:	Vasilis.Vlachoudis@cern.ch
# Date:	04-Sep-2014

# Greek characters http://en.wikipedia.org/wiki/List_of_Unicode_characters
GREEK_NUMERAL_SIGN                                  = u"\u0374"
GREEK_LOWER_NUMERAL_SIGN                            = u"\u0375"
GREEK_YPOGEGRAMMENI                                 = u"\u037A"
GREEK_SMALL_REVERSED_LUNATE_SIGMA_SYMBOL            = u"\u037B"
GREEK_SMALL_DOTTED_LUNATE_SIGMA_SYMBOL              = u"\u037C"
GREEK_SMALL_REVERSED_DOTTED_LUNATE_SIGMA_SYMBOL     = u"\u037D"
GREEK_QUESTION_MARK                                 = u"\u037E"
GREEK_ACUTE_ACCENT                                  = u"\u0384"
GREEK_DIAERESIS_WITH_ACUTE_ACCENT                   = u"\u0385"
GREEK_CAPITAL_A_WITH_ACUTE_ACCENT                   = u"\u0386"
GREEK_ANO_TELEIA                                    = u"\u0387"
GREEK_CAPITAL_EPSILON_WITH_ACUTE_ACCENT             = u"\u0388"
GREEK_CAPITAL_ETA_WITH_ACUTE_ACCENT                 = u"\u0389"
GREEK_CAPITAL_IOTA_WITH_ACUTE_ACCENT                = u"\u038A"
GREEK_CAPITAL_OMICRON_WITH_ACUTE_ACCENT             = u"\u038C"
GREEK_CAPITAL_UPSILON_WITH_ACUTE_ACCENT             = u"\u038E"
GREEK_CAPITAL_OMEGA_WITH_ACUTE_ACCENT               = u"\u038F"
GREEK_SMALL_IOTA_WITH_DIAERESIS_AND_ACUTE_ACCENT    = u"\u0390"
GREEK_CAPITAL_ALPHA                                 = u"\u0391"
GREEK_CAPITAL_BETA                                  = u"\u0392"
GREEK_CAPITAL_GAMMA                                 = u"\u0393"
GREEK_CAPITAL_DELTA                                 = u"\u0394"
GREEK_CAPITAL_EPSILON                               = u"\u0395"
GREEK_CAPITAL_ZETA                                  = u"\u0396"
GREEK_CAPITAL_ETA                                   = u"\u0397"
GREEK_CAPITAL_THETA                                 = u"\u0398"
GREEK_CAPITAL_IOTA                                  = u"\u0399"
GREEK_CAPITAL_KAPPA                                 = u"\u039A"
GREEK_CAPITAL_LAMBDA                                = u"\u039B"
GREEK_CAPITAL_MU                                    = u"\u039C"
GREEK_CAPITAL_NU                                    = u"\u039D"
GREEK_CAPITAL_XI                                    = u"\u039E"
GREEK_CAPITAL_OMICRON                               = u"\u039F"
GREEK_CAPITAL_PI                                    = u"\u03A0"
GREEK_CAPITAL_RHO                                   = u"\u03A1"
GREEK_CAPITAL_SIGMA                                 = u"\u03A3"
GREEK_CAPITAL_TAU                                   = u"\u03A4"
GREEK_CAPITAL_UPSILON                               = u"\u03A5"
GREEK_CAPITAL_PHI                                   = u"\u03A6"
GREEK_CAPITAL_CHI                                   = u"\u03A7"
GREEK_CAPITAL_PSI                                   = u"\u03A8"
GREEK_CAPITAL_OMEGA                                 = u"\u03A9"
GREEK_CAPITAL_IOTA_WITH_DIAERESIS                   = u"\u03AA"
GREEK_CAPITAL_UPSILON_WITH_DIAERESIS                = u"\u03AB"
GREEK_SMALL_ALPHA_WITH_ACUTE_ACCENT                 = u"\u03AC"
GREEK_SMALL_EPSILON_WITH_ACUTE_ACCENT               = u"\u03AD"
GREEK_SMALL_ETA_WITH_ACUTE_ACCENT                   = u"\u03AE"
GREEK_SMALL_IOTA_WITH_ACUTE_ACCENT                  = u"\u03AF"
GREEK_SMALL_UPSILON_WITH_DIAERESIS_AND_ACUTE_ACCENT = u"\u03B0"
GREEK_SMALL_ALPHA                                   = u"\u03B1"
GREEK_SMALL_BETA                                    = u"\u03B2"
GREEK_SMALL_GAMMA                                   = u"\u03B3"
GREEK_SMALL_DELTA                                   = u"\u03B4"
GREEK_SMALL_EPSILON                                 = u"\u03B5"
GREEK_SMALL_ZETA                                    = u"\u03B6"
GREEK_SMALL_ETA                                     = u"\u03B7"
GREEK_SMALL_THETA                                   = u"\u03B8"
GREEK_SMALL_IOTA                                    = u"\u03B9"
GREEK_SMALL_KAPPA                                   = u"\u03BA"
GREEK_SMALL_LAMBDA                                  = u"\u03BB"
GREEK_SMALL_MU                                      = u"\u03BC"
GREEK_SMALL_NU                                      = u"\u03BD"
GREEK_SMALL_XI                                      = u"\u03BE"
GREEK_SMALL_OMICRON                                 = u"\u03BF"
GREEK_SMALL_PI                                      = u"\u03C0"
GREEK_SMALL_RHO                                     = u"\u03C1"
GREEK_SMALL_FINAL_SIGMA                             = u"\u03C2"
GREEK_SMALL_SIGMA                                   = u"\u03C3"
GREEK_SMALL_TAU                                     = u"\u03C4"
GREEK_SMALL_UPSILON                                 = u"\u03C5"
GREEK_SMALL_PHI                                     = u"\u03C6"
GREEK_SMALL_CHI                                     = u"\u03C7"
GREEK_SMALL_PSI                                     = u"\u03C8"
GREEK_SMALL_OMEGA                                   = u"\u03C9"
GREEK_SMALL_IOTA_WITH_DIAERESIS                     = u"\u03CA"
GREEK_SMALL_UPSILON_WITH_DIAERESIS                  = u"\u03CB"
GREEK_SMALL_OMICRON_WITH_ACUTE_ACCENT               = u"\u03CC"
GREEK_SMALL_UPSILON_WITH_ACUTE_ACCENT               = u"\u03CD"
GREEK_SMALL_OMEGA_WITH_ACUTE_ACCENT                 = u"\u03CE"
GREEK_BETA_SYMBOL                                   = u"\u03D0"
GREEK_THETA_SYMBOL                                  = u"\u03D1"
GREEK_UPSILON_WITH_HOOK_SYMBOL                      = u"\u03D2"
GREEK_UPSILON_WITH_ACUTE_AND_HOOK_SYMBOL            = u"\u03D3"
GREEK_UPSILON_WITH_DIAERESIS_AND_HOOK_SYMBOL        = u"\u03D4"
GREEK_PHI_SYMBOL                                    = u"\u03D5"
GREEK_PI_SYMBOL                                     = u"\u03D6"
GREEK_KAI_SYMBOL                                    = u"\u03D7"
GREEK_QOPPA                                         = u"\u03D8"
GREEK_SMALL_QOPPA                                   = u"\u03D9"
GREEK_STIGMA_LETTER                                 = u"\u03DA"
GREEK_SMALL_STIGMA                                  = u"\u03DB"
GREEK_DIGAMMA                                       = u"\u03DC"
GREEK_SMALL_DIGAMMA                                 = u"\u03DD"
GREEK_KOPPA                                         = u"\u03DE"
GREEK_SMALL_KOPPA                                   = u"\u03DF"
GREEK_SAMPI                                         = u"\u03E0"
GREEK_SMALL_SAMPI                                   = u"\u03E1"
COPTIC_CAPITAL_SHEI                                 = u"\u03E2"
COPTIC_SMALL_SHEI                                   = u"\u03E3"
COPTIC_CAPITAL_FEI                                  = u"\u03E4"
COPTIC_SMALL_FEI                                    = u"\u03E5"
COPTIC_CAPITAL_KHEI                                 = u"\u03E6"
COPTIC_SMALL_KHEI                                   = u"\u03E7"
COPTIC_CAPITAL_HORI                                 = u"\u03E8"
COPTIC_SMALL_HORI                                   = u"\u03E9"
COPTIC_CAPITAL_GANGIA                               = u"\u03EA"
COPTIC_SMALL_GANGIA                                 = u"\u03EB"
COPTIC_CAPITAL_SHIMA                                = u"\u03EC"
COPTIC_SMALL_SHIMA                                  = u"\u03ED"
COPTIC_CAPITAL_DEI                                  = u"\u03EE"
COPTIC_SMALL_DEI                                    = u"\u03EF"
GREEK_KAPPA_SYMBOL                                  = u"\u03F0"
GREEK_RHO_SYMBOL                                    = u"\u03F1"
GREEK_LUNATE_SIGMA_SYMBOL                           = u"\u03F2"
GREEK_YOT                                           = u"\u03F3"
GREEK_CAPITAL_THETA_SYMBOL                          = u"\u03F4"
GREEK_LUNATE_EPSILON_SYMBOL                         = u"\u03F5"
GREEK_REVERSED_LUNATE_EPSILON_SYMBOL                = u"\u03F6"
GREEK_CAPITAL_SHO                                   = u"\u03F7"
GREEK_SMALL_SHO                                     = u"\u03F8"
GREEK_CAPITAL_LUNATE_SIGMA_SYMBOL                   = u"\u03F9"
GREEK_CAPITAL_SAN                                   = u"\u03FA"
GREEK_SMALL_SAN                                     = u"\u03FB"
GREEK_RHO_WITH_STROKE_SYMBOL                        = u"\u03FC"
GREEK_CAPITAL_REVERSED_LUNATE_SIGMA_SYMBOL          = u"\u03FD"
GREEK_CAPITAL_DOTTED_LUNATE_SIGMA_SYMBOL            = u"\u03FE"
GREEK_CAPITAL_REVERSED_DOTTED_LUNATE_SIGMA_SYMBOL   = u"\u03FF"

# Geometric Shapes http://en.wikipedia.org/wiki/Geometric_Shapes
BLACK_SQUARE                                  = u"\u25A0"
WHITE_SQUARE                                  = u"\u25A1"
WHITE_SQUARE_WITH_ROUNDED_CORNERS             = u"\u25A2"
WHITE_SQUARE_CONTAINING_BLACK_SMALL_SQUARE    = u"\u25A3"
SQUARE_WITH_HORIZONTAL_FILL                   = u"\u25A4"
SQUARE_WITH_VERTICAL_FILL                     = u"\u25A5"
SQUARE_WITH_ORTHOGONAL_CROSSHATCH_FILL        = u"\u25A6"
SQUARE_WITH_UPPER_LEFT_TO_LOWER_RIGHT_FILL    = u"\u25A7"
SQUARE_WITH_UPPER_RIGHT_TO_LOWER_LEFT_FILL    = u"\u25A8"
SQUARE_WITH_DIAGONAL_CROSSHATCH_FILL          = u"\u25A9"
BLACK_SMALL_SQUARE                            = u"\u25AA"
WHITE_SMALL_SQUARE                            = u"\u25AB"
BLACK_RECTANGLE                               = u"\u25AC"
WHITE_RECTANGLE                               = u"\u25AD"
BLACK_VERTICAL_RECTANGLE                      = u"\u25AE"
WHITE_VERTICAL_RECTANGLE                      = u"\u25AF"
BLACK_PARALLELOGRAM                           = u"\u25B0"
WHITE_PARALLELOGRAM                           = u"\u25B1"
BLACK_UP_POINTING_TRIANGLE                    = u"\u25B2"
WHITE_UP_POINTING_TRIANGLE                    = u"\u25B3"
BLACK_UP_POINTING_SMALL_TRIANGLE              = u"\u25B4"
WHITE_UP_POINTING_SMALL_TRIANGLE              = u"\u25B5"
BLACK_RIGHT_POINTING_TRIANGLE                 = u"\u25B6"
WHITE_RIGHT_POINTING_TRIANGLE                 = u"\u25B7"
BLACK_RIGHT_POINTING_SMALL_TRIANGLE           = u"\u25B8"
WHITE_RIGHT_POINTING_SMALL_TRIANGLE           = u"\u25B9"
BLACK_RIGHT_POINTING_POINTER                  = u"\u25BA"
WHITE_RIGHT_POINTING_POINTER                  = u"\u25BB"
BLACK_DOWN_POINTING_TRIANGLE                  = u"\u25BC"
WHITE_DOWN_POINTING_TRIANGLE                  = u"\u25BD"
BLACK_DOWN_POINTING_SMALL_TRIANGLE            = u"\u25BE"
WHITE_DOWN_POINTING_SMALL_TRIANGLE            = u"\u25BF"
BLACK_LEFT_POINTING_TRIANGLE                  = u"\u25C0"
WHITE_LEFT_POINTING_TRIANGLE                  = u"\u25C1"
BLACK_LEFT_POINTING_SMALL_TRIANGLE            = u"\u25C2"
WHITE_LEFT_POINTING_SMALL_TRIANGLE            = u"\u25C3"
BLACK_LEFT_POINTING_POINTER                   = u"\u25C4"
WHITE_LEFT_POINTING_POINTER                   = u"\u25C5"
BLACK_DIAMOND                                 = u"\u25C6"
WHITE_DIAMOND                                 = u"\u25C7"
WHITE_DIAMOND_CONTAINING_BLACK_SMALL_DIAMOND  = u"\u25C8"
FISHEYE                                       = u"\u25C9"
LOZENGE                                       = u"\u25CA"
WHITE_CIRCLE                                  = u"\u25CB"
DOTTED_CIRCLE                                 = u"\u25CC"
CIRCLE_WITH_VERTICAL_FILL                     = u"\u25CD"
BULLSEYE                                      = u"\u25CE"
BLACK_CIRCLE                                  = u"\u25CF"
CIRCLE_WITH_LEFT_HALF_BLACK                   = u"\u25D0"
CIRCLE_WITH_RIGHT_HALF_BLACK                  = u"\u25D1"
CIRCLE_WITH_LOWER_HALF_BLACK                  = u"\u25D2"
CIRCLE_WITH_UPPER_HALF_BLACK                  = u"\u25D3"
CIRCLE_WITH_UPPER_RIGHT_QUADRANT_BLACK        = u"\u25D4"
CIRCLE_WITH_ALL_BUT_UPPER_LEFT_QUADRANT_BLACK = u"\u25D5"
LEFT_HALF_BLACK_CIRCLE                        = u"\u25D6"
RIGHT_HALF_BLACK_CIRCLE                       = u"\u25D7"
INVERSE_BULLET                                = u"\u25D8"
INVERSE_WHITE_CIRCLE                          = u"\u25D9"
UPPER_HALF_INVERSE_WHITE_CIRCLE               = u"\u25DA"
LOWER_HALF_INVERSE_WHITE_CIRCLE               = u"\u25DB"
UPPER_LEFT_QUADRANT_CIRCULAR_ARC              = u"\u25DC"
UPPER_RIGHT_QUADRANT_CIRCULAR_ARC             = u"\u25DD"
LOWER_RIGHT_QUADRANT_CIRCULAR_ARC             = u"\u25DE"
LOWER_LEFT_QUADRANT_CIRCULAR_ARC              = u"\u25DF"
UPPER_HALF_CIRCLE                             = u"\u25E0"
LOWER_HALF_CIRCLE                             = u"\u25E1"
BLACK_LOWER_RIGHT_TRIANGLE                    = u"\u25E2"
BLACK_LOWER_LEFT_TRIANGLE                     = u"\u25E3"
BLACK_UPPER_LEFT_TRIANGLE                     = u"\u25E4"
BLACK_UPPER_RIGHT_TRIANGLE                    = u"\u25E5"
WHITE_BULLET                                  = u"\u25E6"
SQUARE_WITH_LEFT_HALF_BLACK                   = u"\u25E7"
SQUARE_WITH_RIGHT_HALF_BLACK                  = u"\u25E8"
SQUARE_WITH_UPPER_LEFT_DIAGONAL_HALF_BLACK    = u"\u25E9"
SQUARE_WITH_LOWER_RIGHT_DIAGONAL_HALF_BLACK   = u"\u25EA"
WHITE_SQUARE_WITH_VERTICAL_BISECTING_LINE     = u"\u25EB"
WHITE_UP_POINTING_TRIANGLE_WITH_DOT           = u"\u25EC"
UP_POINTING_TRIANGLE_WITH_LEFT_HALF_BLACK     = u"\u25ED"
UP_POINTING_TRIANGLE_WITH_RIGHT_HALF_BLACK    = u"\u25EE"
LARGE_CIRCLE                                  = u"\u25EF"
WHITE_SQUARE_WITH_UPPER_LEFT_QUADRANT         = u"\u25F0"
WHITE_SQUARE_WITH_LOWER_LEFT_QUADRANT         = u"\u25F1"
WHITE_SQUARE_WITH_LOWER_RIGHT_QUADRANT        = u"\u25F2"
WHITE_SQUARE_WITH_UPPER_RIGHT_QUADRANT        = u"\u25F3"
WHITE_CIRCLE_WITH_UPPER_LEFT_QUADRANT         = u"\u25F4"
WHITE_CIRCLE_WITH_LOWER_LEFT_QUADRANT         = u"\u25F5"
WHITE_CIRCLE_WITH_LOWER_RIGHT_QUADRANT        = u"\u25F6"
WHITE_CIRCLE_WITH_UPPER_RIGHT_QUADRANT        = u"\u25F7"
UPPER_LEFT_TRIANGLE                           = u"\u25F8"
UPPER_RIGHT_TRIANGLE                          = u"\u25F9"
LOWER_LEFT_TRIANGLE                           = u"\u25FA"
WHITE_MEDIUM_SQUARE                           = u"\u25FB"
BLACK_MEDIUM_SQUARE                           = u"\u25FC"
WHITE_MEDIUM_SMALL_SQUARE                     = u"\u25FD"
BLACK_MEDIUM_SMALL_SQUARE                     = u"\u25FE"
LOWER_RIGHT_TRIANGLE                          = u"\u25FF"

# Arrows (http://en.wikipedia.org/wiki/Arrow_%28symbol%29)
LEFTWARDS_ARROW                                     = u"\u2190"
UPWARDS_ARROW                                       = u"\u2191"
RIGHTWARDS_ARROW                                    = u"\u2192"
DOWNWARDS_ARROW                                     = u"\u2193"
LEFT_RIGHT_ARROW                                    = u"\u2194"
UP_DOWN_ARROW                                       = u"\u2195"
NORTH_WEST_ARROW                                    = u"\u2196"
NORTH_EAST_ARROW                                    = u"\u2197"
SOUTH_EAST_ARROW                                    = u"\u2198"
SOUTH_WEST_ARROW                                    = u"\u2199"
LEFTWARDS_ARROW_WITH_STROKE                         = u"\u219A"
RIGHTWARDS_ARROW_WITH_STROKE                        = u"\u219B"
LEFTWARDS_WAVE_ARROW                                = u"\u219C"
RIGHTWARDS_WAVE_ARROW                               = u"\u219D"
LEFTWARDS_TWO_HEADED_ARROW                          = u"\u219E"
UPWARDS_TWO_HEADED_ARROW                            = u"\u219F"
RIGHTWARDS_TWO_HEADED_ARROW                         = u"\u21A0"
DOWNWARDS_TWO_HEADED_ARROW                          = u"\u21A1"
LEFTWARDS_ARROW_WITH_TAIL                           = u"\u21A2"
RIGHTWARDS_ARROW_WITH_TAIL                          = u"\u21A3"
LEFTWARDS_ARROW_FROM_BAR                            = u"\u21A4"
UPWARDS_ARROW_FROM_BAR                              = u"\u21A5"
RIGHTWARDS_ARROW_FROM_BAR                           = u"\u21A6"
DOWNWARDS_ARROW_FROM_BAR                            = u"\u21A7"
UP_DOWN_ARROW_WITH_BASE                             = u"\u21A8"
LEFTWARDS_ARROW_WITH_HOOK                           = u"\u21A9"
RIGHTWARDS_ARROW_WITH_HOOK                          = u"\u21AA"
LEFTWARDS_ARROW_WITH_LOOP                           = u"\u21AB"
RIGHTWARDS_ARROW_WITH_LOOP                          = u"\u21AC"
LEFT_RIGHT_WAVE_ARROW                               = u"\u21AD"
LEFT_RIGHT_ARROW_WITH_STROKE                        = u"\u21AE"
DOWNWARDS_ZIGZAG_ARROW                              = u"\u21AF"
UPWARDS_ARROW_WITH_TIP_LEFTWARDS                    = u"\u21B0"
UPWARDS_ARROW_WITH_TIP_RIGHTWARDS                   = u"\u21B1"
DOWNWARDS_ARROW_WITH_TIP_LEFTWARDS                  = u"\u21B2"
DOWNWARDS_ARROW_WITH_TIP_RIGHTWARDS                 = u"\u21B3"
RIGHTWARDS_ARROW_WITH_CORNER_DOWNWARDS              = u"\u21B4"
DOWNWARDS_ARROW_WITH_CORNER_LEFTWARDS               = u"\u21B5"
ANTICLOCKWISE_TOP_SEMICIRCLE_ARROW                  = u"\u21B6"
CLOCKWISE_TOP_SEMICIRCLE_ARROW                      = u"\u21B7"
NORTH_WEST_ARROW_TO_LONG_BAR                        = u"\u21B8"
LEFTWARDS_ARROW_TO_BAR_OVER_RIGHTWARDS_ARROW_TO_BAR = u"\u21B9"
ANTICLOCKWISE_OPEN_CIRCLE_ARROW                     = u"\u21BA"
CLOCKWISE_OPEN_CIRCLE_ARROW                         = u"\u21BB"
LEFTWARDS_HARPOON_WITH_BARB_UPWARDS                 = u"\u21BC"
LEFTWARDS_HARPOON_WITH_BARB_DOWNWARDS               = u"\u21BD"
UPWARDS_HARPOON_WITH_BARB_RIGHTWARDS                = u"\u21BE"
UPWARDS_HARPOON_WITH_BARB_LEFTWARDS                 = u"\u21BF"
RIGHTWARDS_HARPOON_WITH_BARB_UPWARDS                = u"\u21C0"
RIGHTWARDS_HARPOON_WITH_BARB_DOWNWARDS              = u"\u21C1"
DOWNWARDS_HARPOON_WITH_BARB_RIGHTWARDS              = u"\u21C2"
DOWNWARDS_HARPOON_WITH_BARB_LEFTWARDS               = u"\u21C3"
RIGHTWARDS_ARROW_OVER_LEFTWARDS_ARROW               = u"\u21C4"
UPWARDS_ARROW_LEFTWARDS_OF_DOWNWARDS_ARROW          = u"\u21C5"
LEFTWARDS_ARROW_OVER_RIGHTWARDS_ARROW               = u"\u21C6"
LEFTWARDS_PAIRED_ARROWS                             = u"\u21C7"
UPWARDS_PAIRED_ARROWS                               = u"\u21C8"
RIGHTWARDS_PAIRED_ARROWS                            = u"\u21C9"
DOWNWARDS_PAIRED_ARROWS                             = u"\u21CA"
LEFTWARDS_HARPOON_OVER_RIGHTWARDS_HARPOON           = u"\u21CB"
RIGHTWARDS_HARPOON_OVER_LEFTWARDS_HARPOON           = u"\u21CC"
LEFTWARDS_DOUBLE_ARROW_WITH_STROKE                  = u"\u21CD"
LEFT_RIGHT_DOUBLE_ARROW_WITH_STROKE                 = u"\u21CE"
RIGHTWARDS_DOUBLE_ARROW_WITH_STROKE                 = u"\u21CF"
LEFTWARDS_DOUBLE_ARROW                              = u"\u21D0"
UPWARDS_DOUBLE_ARROW                                = u"\u21D1"
RIGHTWARDS_DOUBLE_ARROW                             = u"\u21D2"
DOWNWARDS_DOUBLE_ARROW                              = u"\u21D3"
LEFT_RIGHT_DOUBLE_ARROW                             = u"\u21D4"
UP_DOWN_DOUBLE_ARROW                                = u"\u21D5"
NORTH_WEST_DOUBLE_ARROW                             = u"\u21D6"
NORTH_EAST_DOUBLE_ARROW                             = u"\u21D7"
SOUTH_EAST_DOUBLE_ARROW                             = u"\u21D8"
SOUTH_WEST_DOUBLE_ARROW                             = u"\u21D9"
LEFTWARDS_TRIPLE_ARROW                              = u"\u21DA"
RIGHTWARDS_TRIPLE_ARROW                             = u"\u21DB"
LEFTWARDS_SQUIGGLE_ARROW                            = u"\u21DC"
RIGHTWARDS_SQUIGGLE_ARROW                           = u"\u21DD"
UPWARDS_ARROW_WITH_DOUBLE_STROKE                    = u"\u21DE"
DOWNWARDS_ARROW_WITH_DOUBLE_STROKE                  = u"\u21DF"
LEFTWARDS_DASHED_ARROW                              = u"\u21E0"
UPWARDS_DASHED_ARROW                                = u"\u21E1"
RIGHTWARDS_DASHED_ARROW                             = u"\u21E2"
DOWNWARDS_DASHED_ARROW                              = u"\u21E3"
LEFTWARDS_ARROW_TO_BAR                              = u"\u21E4"
RIGHTWARDS_ARROW_TO_BAR                             = u"\u21E5"
LEFTWARDS_THICK_ARROW                               = u"\u21E6"
UPWARDS_THICK_ARROW                                 = u"\u21E7"
RIGHTWARDS_THICK_ARROW                              = u"\u21E8"
DOWNWARDS_THICK_ARROW                               = u"\u21E9"
UPWARDS_THICK_ARROW_FROM_BAR                        = u"\u21EA"
UPWARDS_THICK_ARROW_ON_PEDESTAL                     = u"\u21EB"
UPWARDS_THICK_ARROW_ON_PEDESTAL_WITH_HORIZONTAL_BAR = u"\u21EC"
UPWARDS_THICK_ARROW_ON_PEDESTAL_WITH_VERTICAL_BAR   = u"\u21ED"
UPWARDS_THICK_DOUBLE_ARROW                          = u"\u21EE"
UPWARDS_THICK_DOUBLE_ARROW_ON_PEDESTAL              = u"\u21EF"
RIGHTWARDS_THICK_ARROW_FROM_WALL                    = u"\u21F0"
NORTH_WEST_ARROW_TO_CORNER                          = u"\u21F1"
SOUTH_EAST_ARROW_TO_CORNER                          = u"\u21F2"
UP_DOWN_THICK_ARROW                                 = u"\u21F3"
RIGHT_ARROW_WITH_SMALL_CIRCLE                       = u"\u21F4"
DOWNWARDS_ARROW_LEFTWARDS_OF_UPWARDS_ARROW          = u"\u21F5"
THREE_RIGHTWARDS_ARROWS                             = u"\u21F6"
LEFTWARDS_ARROW_WITH_VERTICAL_STROKE                = u"\u21F7"
RIGHTWARDS_ARROW_WITH_VERTICAL_STROKE               = u"\u21F8"
LEFT_RIGHT_ARROW_WITH_VERTICAL_STROKE               = u"\u21F9"
LEFTWARDS_ARROW_WITH_DOUBLE_VERTICAL_STROKE         = u"\u21FA"
RIGHTWARDS_ARROW_WITH_DOUBLE_VERTICAL_STROKE        = u"\u21FB"
LEFT_RIGHT_ARROW_WITH_DOUBLE_VERTICAL_STROKE        = u"\u21FC"
LEFTWARDS_OPEN_HEADED_ARROW                         = u"\u21FD"
RIGHTWARDS_OPEN_HEADED_ARROW                        = u"\u21FE"
LEFT_RIGHT_OPEN_HEADED_ARROW                        = u"\u21FF"

# Miscellaneous Symbols
BLACK_SUN_WITH_RAYS                              = u"\u2600"
CLOUD                                            = u"\u2601"
UMBRELLA                                         = u"\u2602"
SNOWMAN                                          = u"\u2603"
COMET                                            = u"\u2604"
BLACK_STAR                                       = u"\u2605"
STAR                                             = u"\u2606"
LIGHTNING                                        = u"\u2607"
THUNDERSTORM                                     = u"\u2608"
SUN                                              = u"\u2609"
ASCENDING_NODE                                   = u"\u260A"
DESCENDING_NODE                                  = u"\u260B"
CONJUNCTION                                      = u"\u260C"
OPPOSITION                                       = u"\u260D"
BLACK_TELEPHONE                                  = u"\u260E"
WHITE_TELEPHONE                                  = u"\u260F"
BALLOT_BOX                                       = u"\u2610"
BALLOT_BOX_WITH_CHECK                            = u"\u2611"
BALLOT_BOX_WITH_X                                = u"\u2612"
SALTIRE                                          = u"\u2613"
UMBRELLA_WITH_RAINDROPS                          = u"\u2614"
HOT_BEVERAGE                                     = u"\u2615"
WHITE_SHOGI_PIECE                                = u"\u2616"
BLACK_SHOGI_PIECE                                = u"\u2617"
SHAMROCK                                         = u"\u2618"
REVERSED_ROTATED_FLORAL_HEART_BULLET             = u"\u2619"
BLACK_LEFT_POINTING_INDEX                        = u"\u261A"
BLACK_RIGHT_POINTING_INDEX                       = u"\u261B"
WHITE_LEFT_POINTING_INDEX                        = u"\u261C"
WHITE_UP_POINTING_INDEX                          = u"\u261D"
WHITE_RIGHT_POINTING_INDEX                       = u"\u261E"
WHITE_DOWN_POINTING_INDEX                        = u"\u261F"
SKULL_AND_CROSSBONES                             = u"\u2620"
CAUTION_SIGN                                     = u"\u2621"
RADIOACTIVE_SIGN                                 = u"\u2622"
BIOHAZARD_SIGN                                   = u"\u2623"
CADUCEUS                                         = u"\u2624"
ANKH                                             = u"\u2625"
RUSSIAN_CROSS                                    = u"\u2626"
CHI_RHO                                          = u"\u2627"
CROSS_OF_LORRAINE                                = u"\u2628"
CROSS_OF_JERUSALEM                               = u"\u2629"
STAR_AND_CRESCENT                                = u"\u262A"
FARSI_SYMBOL                                     = u"\u262B"
ADI_SHAKTI                                       = u"\u262C"
HAMMER_AND_SICKLE                                = u"\u262D"
PEACE_SYMBOL                                     = u"\u262E"
YIN_AND_YANG                                     = u"\u262F"
TRIGRAM_FOR_HEAVEN                               = u"\u2630"
TRIGRAM_FOR_LAKE                                 = u"\u2631"
TRIGRAM_FOR_FIRE                                 = u"\u2632"
TRIGRAM_FOR_THUNDER                              = u"\u2633"
TRIGRAM_FOR_WIND                                 = u"\u2634"
TRIGRAM_FOR_WATER                                = u"\u2635"
TRIGRAM_FOR_MOUNTAIN                             = u"\u2636"
TRIGRAM_FOR_EARTH                                = u"\u2637"
WHEEL_OF_DHARMA                                  = u"\u2638"
WHITE_FROWNING_FACE                              = u"\u2639"
WHITE_SMILING_FACE                               = u"\u263A"
BLACK_SMILING_FACE                               = u"\u263B"
WHITE_SUN_WITH_RAYS                              = u"\u263C"
FIRST_QUARTER_MOON                               = u"\u263D"
LAST_QUARTER_MOON                                = u"\u263E"
MERCURY                                          = u"\u263F"
FEMALE_SIGN                                      = u"\u2640"
EARTH                                            = u"\u2641"
MALE_SIGN                                        = u"\u2642"
JUPITER                                          = u"\u2643"
SATURN                                           = u"\u2644"
URANUS                                           = u"\u2645"
NEPTUNE                                          = u"\u2646"
PLUTO                                            = u"\u2647"
ARIES                                            = u"\u2648"
TAURUS                                           = u"\u2649"
GEMINI                                           = u"\u264A"
CANCER                                           = u"\u264B"
LEO                                              = u"\u264C"
VIRGO                                            = u"\u264D"
LIBRA                                            = u"\u264E"
SCORPIUS                                         = u"\u264F"
SAGITTARIUS                                      = u"\u2650"
CAPRICORN                                        = u"\u2651"
AQUARIUS                                         = u"\u2652"
PISCES                                           = u"\u2653"
WHITE_CHESS_KING                                 = u"\u2654"
WHITE_CHESS_QUEEN                                = u"\u2655"
WHITE_CHESS_ROOK                                 = u"\u2656"
WHITE_CHESS_BISHOP                               = u"\u2657"
WHITE_CHESS_KNIGHT                               = u"\u2658"
WHITE_CHESS_PAWN                                 = u"\u2659"
BLACK_CHESS_KING                                 = u"\u265A"
BLACK_CHESS_QUEEN                                = u"\u265B"
BLACK_CHESS_ROOK                                 = u"\u265C"
BLACK_CHESS_BISHOP                               = u"\u265D"
BLACK_CHESS_KNIGHT                               = u"\u265E"
BLACK_CHESS_PAWN                                 = u"\u265F"
BLACK_SPADE_SUIT                                 = u"\u2660"
WHITE_HEART_SUIT                                 = u"\u2661"
WHITE_DIAMOND_SUIT                               = u"\u2662"
BLACK_CLUB_SUIT                                  = u"\u2663"
WHITE_SPADE_SUIT                                 = u"\u2664"
BLACK_HEART_SUIT                                 = u"\u2665"
BLACK_DIAMOND_SUIT                               = u"\u2666"
WHITE_CLUB_SUIT                                  = u"\u2667"
HOT_SPRING                                       = u"\u2668"
QUARTER_NOTE                                     = u"\u2669"
EIGHTH_NOTE                                      = u"\u266A"
BEAMED_EIGHTH_NOTES                              = u"\u266B"
BEAMED_SIXTEENTH_NOTES                           = u"\u266C"
MUSIC_FLAT_SIGN                                  = u"\u266D"
MUSIC_NATURAL_SIGN                               = u"\u266E"
MUSIC_SHARP_SIGN                                 = u"\u266F"
WEST_SYRIAC_CROSS                                = u"\u2670"
EAST_SYRIAC_CROSS                                = u"\u2671"
UNIVERSAL_RECYCLING_SYMBOL                       = u"\u2672"
RECYCLING_SYMBOL_FOR_TYPE_1_PLASTICS             = u"\u2673"
RECYCLING_SYMBOL_FOR_TYPE_2_PLASTICS             = u"\u2674"
RECYCLING_SYMBOL_FOR_TYPE_3_PLASTICS             = u"\u2675"
RECYCLING_SYMBOL_FOR_TYPE_4_PLASTICS             = u"\u2676"
RECYCLING_SYMBOL_FOR_TYPE_5_PLASTICS             = u"\u2677"
RECYCLING_SYMBOL_FOR_TYPE_6_PLASTICS             = u"\u2678"
RECYCLING_SYMBOL_FOR_TYPE_7_PLASTICS             = u"\u2679"
RECYCLING_SYMBOL_FOR_GENERIC_MATERIALS           = u"\u267A"
BLACK_UNIVERSAL_RECYCLING_SYMBOL                 = u"\u267B"
RECYCLED_PAPER_SYMBOL                            = u"\u267C"
PARTIALLY_RECYCLED_PAPER_SYMBOL                  = u"\u267D"
PERMANENT_PAPER_SIGN                             = u"\u267E"
WHEELCHAIR_SYMBOL                                = u"\u267F"
DIE_FACE_1                                       = u"\u2680"
DIE_FACE_2                                       = u"\u2681"
DIE_FACE_3                                       = u"\u2682"
DIE_FACE_4                                       = u"\u2683"
DIE_FACE_5                                       = u"\u2684"
DIE_FACE_6                                       = u"\u2685"
WHITE_CIRCLE_WITH_DOT_RIGHT                      = u"\u2686"
WHITE_CIRCLE_WITH_TWO_DOTS                       = u"\u2687"
BLACK_CIRCLE_WITH_WHITE_DOT_RIGHT                = u"\u2688"
BLACK_CIRCLE_WITH_TWO_WHITE_DOTS                 = u"\u2689"
MONOGRAM_FOR_YANG                                = u"\u268A"
MONOGRAM_FOR_YIN                                 = u"\u268B"
DIGRAM_FOR_GREATER_YANG                          = u"\u268C"
DIGRAM_FOR_LESSER_YIN                            = u"\u268D"
DIGRAM_FOR_LESSER_YANG                           = u"\u268E"
DIGRAM_FOR_GREATER_YIN                           = u"\u268F"
WHITE_FLAG                                       = u"\u2690"
BLACK_FLAG                                       = u"\u2691"
HAMMER_AND_PICK                                  = u"\u2692"
ANCHOR                                           = u"\u2693"
CROSSED_SWORDS                                   = u"\u2694"
STAFF_OF_AESCULAPIUS                             = u"\u2695"
WEIGHING_SCALES                                  = u"\u2696"
ALEMBIC                                          = u"\u2697"
FLOWER                                           = u"\u2698"
GEAR                                             = u"\u2699"
STAFF_OF_HERMES                                  = u"\u269A"
ATOM_SYMBOL                                      = u"\u269B"
FLEUR_DE_LIS                                     = u"\u269C"
OUTLINED_WHITE_STAR                              = u"\u269D"
THREE_LINES_CONVERGING_RIGHT                     = u"\u269E"
THREE_LINES_CONVERGING_LEFT                      = u"\u269F"
WARNING_SIGN                                     = u"\u26A0"
HIGH_VOLTAGE_SIGN                                = u"\u26A1"
DOUBLED_FEMALE_SIGN                              = u"\u26A2"
DOUBLED_MALE_SIGN                                = u"\u26A3"
INTERLOCKED_MALE_AND_FEMALE_SIGN                 = u"\u26A4"
MALE_AND_FEMALE_SIGN                             = u"\u26A5"
MALE_WITH_STROKE_SIGN                            = u"\u26A6"
MALE_WITH_STROKE_AND_MALE_AND_FEMALE_SIGN        = u"\u26A7"
VERTICAL_MALE_WITH_STROKE_SIGN                   = u"\u26A8"
HORIZONTAL_MALE_WITH_STROKE_SIGN                 = u"\u26A9"
MEDIUM_WHITE_CIRCLE                              = u"\u26AA"
MEDIUM_BLACK_CIRCLE                              = u"\u26AB"
MEDIUM_SMALL_WHITE_CIRCLE                        = u"\u26AC"
MARRIAGE_SYMBOL                                  = u"\u26AD"
DIVORCE_SYMBOL                                   = u"\u26AE"
UNMARRIED_PARTNERSHIP_SYMBOL                     = u"\u26AF"
COFFIN                                           = u"\u26B0"
FUNERAL_URN                                      = u"\u26B1"
NEUTER                                           = u"\u26B2"
CERES                                            = u"\u26B3"
PALLAS                                           = u"\u26B4"
JUNO                                             = u"\u26B5"
VESTA                                            = u"\u26B6"
CHIRON                                           = u"\u26B7"
BLACK_MOON_LILITH                                = u"\u26B8"
SEXTILE                                          = u"\u26B9"
SEMISEXTILE                                      = u"\u26BA"
QUINCUNX                                         = u"\u26BB"
SESQUIQUADRATE                                   = u"\u26BC"
SOCCER_BALL                                      = u"\u26BD"
BASEBALL                                         = u"\u26BE"
SQUARED_KEY                                      = u"\u26BF"
WHITE_DRAUGHTS_MAN                               = u"\u26C0"
WHITE_DRAUGHTS_KING                              = u"\u26C1"
BLACK_DRAUGHTS_MAN                               = u"\u26C2"
BLACK_DRAUGHTS_KING                              = u"\u26C3"
SNOWMAN_WITHOUT_SNOW                             = u"\u26C4"
SUN_BEHIND_CLOUD                                 = u"\u26C5"
RAIN                                             = u"\u26C6"
BLACK_SNOWMAN                                    = u"\u26C7"
THUNDER_CLOUD_AND_RAIN                           = u"\u26C8"
TURNED_WHITE_SHOGI_PIECE                         = u"\u26C9"
TURNED_BLACK_SHOGI_PIECE                         = u"\u26CA"
WHITE_DIAMOND_IN_SQUARE                          = u"\u26CB"
CROSSING_LANES                                   = u"\u26CC"
DISABLED_CAR                                     = u"\u26CD"
OPHIUCHUS                                        = u"\u26CE"
PICK                                             = u"\u26CF"
CAR_SLIDING                                      = u"\u26D0"
HELMET_WITH_WHITE_CROSS                          = u"\u26D1"
CIRCLED_CROSSING_LANES                           = u"\u26D2"
CHAINS                                           = u"\u26D3"
NO_ENTRY                                         = u"\u26D4"
ALTERNATE_ONE_WAY_LEFT_WAY_TRAFFIC               = u"\u26D5"
BLACK_TWO_WAY_LEFT_WAY_TRAFFIC                   = u"\u26D6"
WHITE_TWO_WAY_LEFT_WAY_TRAFFIC                   = u"\u26D7"
BLACK_LEFT_LANE_MERGE                            = u"\u26D8"
WHITE_LEFT_LANE_MERGE                            = u"\u26D9"
DRIVE_SLOW_SIGN                                  = u"\u26DA"
HEAVY_WHITE_DOWN_POINTING_TRIANGLE               = u"\u26DB"
LEFT_CLOSED_ENTRY                                = u"\u26DC"
SQUARED_SALTIRE                                  = u"\u26DD"
FALLING_DIAGONAL_IN_WHITE_CIRCLE_IN_BLACK_SQUARE = u"\u26DE"
BLACK_TRUCK                                      = u"\u26DF"
RESTRICTED_LEFT_ENTRY_1                          = u"\u26E0"
RESTRICTED_LEFT_ENTRY_2                          = u"\u26E1"
ASTRONOMICAL_SYMBOL_FOR_URANUS                   = u"\u26E2"
HEAVY_CIRCLE_WITH_STROKE_AND_TWO_DOTS_ABOVE      = u"\u26E3"
PENTAGRAM                                        = u"\u26E4"
RIGHT_HANDED_INTERLACED_PENTAGRAM                = u"\u26E5"
LEFT_HANDED_INTERLACED_PENTAGRAM                 = u"\u26E6"
INVERTED_PENTAGRAM                               = u"\u26E7"
BLACK_CROSS_ON_SHIELD                            = u"\u26E8"
SHINTO_SHRINE                                    = u"\u26E9"
CHURCH                                           = u"\u26EA"
CASTLE                                           = u"\u26EB"
HISTORIC_SITE                                    = u"\u26EC"
GEAR_WITHOUT_HUB                                 = u"\u26ED"
GEAR_WITH_HANDLES                                = u"\u26EE"
MAP_SYMBOL_FOR_LIGHTHOUSE                        = u"\u26EF"
MOUNTAIN                                         = u"\u26F0"
UMBRELLA_ON_GROUND                               = u"\u26F1"
FOUNTAIN                                         = u"\u26F2"
FLAG_IN_HOLE                                     = u"\u26F3"
FERRY                                            = u"\u26F4"
SAILBOAT                                         = u"\u26F5"
SQUARE_FOUR_CORNERS                              = u"\u26F6"
SKIER                                            = u"\u26F7"
ICE_SKATE                                        = u"\u26F8"
PERSON_WITH_BALL                                 = u"\u26F9"
TENT                                             = u"\u26FA"
JAPANESE_BANK_SYMBOL                             = u"\u26FB"
HEADSTONE_GRAVEYARD_SYMBOL                       = u"\u26FC"
FUEL_PUMP                                        = u"\u26FD"
CUP_ON_BLACK_SQUARE                              = u"\u26FE"
WHITE_FLAG_WITH_HORIZONTAL_MIDDLE_BLACK_STRIPE   = u"\u26FF"

# Mathematical symbols
MATH_TIMES                                       = u"\u00D7"
MATH_HEAVY_MULTIPLICATION                        = u"\u2716"
MATH_OSLASH                                      = u"\u00D8"
MATH_UP_TACK                                     = u"\u22A5"



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:  Attila Kolinger
# Date:	   26.feb.2018.

__author__ = "Attila Kolinger"
__email__ = "attila.kolinger@gmail.com"  # <<< here put an email where plugins users can contact you

# Here import the libraries you need, these are necessary to modify the code
from CNC import CNC, Block
from ToolsPage import Plugin
import math


# ==============================================================================
# Manual drillmark plugin
# ==============================================================================
class Tool(Plugin):
	__doc__ = _("""This plugin is for creating drilling marks with a laser engraver for manual drilling""")  # <<< This comment will be show as tooltip for the ribbon button

	def __init__(self, master):
		Plugin.__init__(self, master, "Marker for manual drilling")
		self.icon = "crosshair"
		self.group = "Generator"
		self.variables = [
			("name", "db", "", _("Name")),  # used to store plugin settings in the internal database
			("Mark size", "mm", 10.0, _("Drill mark size")),  # a variable that will be converted in mm/inch based on bCNC settting
			("PosX", "mm", 0, _("Mark X center")),  # an integer variable
			("PosY", "mm", 0, _("Mark Y center")),  # a float value variable
			("Burn time", "float", 4.0, _("Burn time for drillmark")),
			("Burn power", "float", 1000.0, _("Burn power for drillmark")),
			("Mark power", "float", 400.0, _("Mark drawing power")),
			("Mark type", "Point,Cross,Cross45,Star,Spikes,Spikes45,SpikesStar,SpikesStar45", "Cross", _("Type of the mark")),  # a multiple choice combo box
			("Draw ring", "bool", True, _("Ring mark (d/2)"))
		]
		self.sin45 = math.sqrt(0.5)
		self.buttons.append("exe")  # <<< This is the button added at bottom to call the execute method below
		self.help = "This plugin is for creating drilling marks with a laser engraver for manual drilling"

	def appendBurn(self, app, block):
		x0 = self.fromMm("PosX")
		y0 = self.fromMm("PosY")
		movefeed = app.cnc["cutfeed"]
		burntime = self["Burn time"]
		burnpower = self["Burn power"]
		block.append(CNC.grapid(x=x0, y=y0))
		block.append("g1 m3 %s" % (CNC.fmt('s', burnpower)))
		block.append("g4 %s" % (CNC.fmt('p', burntime)))
		block.append("m5")

	def getPowerLine(self, app):
		if CNC.laseradaptive:
			pwrcode = "m4"
		else:
			pwrcode = "m3"
		markpower = self["Mark power"]
		return "%s %s" % (pwrcode, CNC.fmt('s', markpower))

	def appendMark(self, app, block):
		def appendCross(block):
			block.append("m5")
			block.append(CNC.grapid(x=x0, y=y0 + marksizehalf, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.gline(y=y0 - marksizehalf, f=drawfeed))
			block.append("m5")
			block.append(CNC.grapid(x=x0 + marksizehalf, y=y0, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.gline(x=x0 - marksizehalf, f=drawfeed))
			block.append("m5")

		def appendCross45(block):
			msh = marksizehalf * self.sin45
			block.append("m5")
			block.append(CNC.grapid(x=x0 - msh, y=y0 + msh, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.gline(x=x0 + msh, y=y0 - msh, f=drawfeed))
			block.append("m5")
			block.append(CNC.grapid(x=x0 + msh, y=y0 + msh, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.gline(x=x0 - msh, y=y0 - msh, f=drawfeed))
			block.append("m5")

		def appendCircle(block):
			block.append("m5")
			block.append(CNC.grapid(x=x0, y=y0 + marksizehalf / 2, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.garc(2, x=x0, y=y0 + marksizehalf / 2, j=-marksizehalf / 2, i=0, f=drawfeed))
			block.append("m5")

		def appendSpikes(block):
			sinSpike = math.sin(math.atan(2.0 - math.sqrt(3)))
			cosSpike = math.cos(math.atan(2.0 - math.sqrt(3)))
			block.append("m5")
			block.append(CNC.grapid(x=x0, y=y0, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.gline(x=x0 + marksizehalf * sinSpike, y=y0 - marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * sinSpike, y=y0 - marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 + marksizehalf * sinSpike, y=y0 + marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * sinSpike, y=y0 + marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0, y=y0, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * cosSpike, y=y0 + marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * cosSpike, y=y0 - marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 + marksizehalf * cosSpike, y=y0 + marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 + marksizehalf * cosSpike, y=y0 - marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0, y=y0, f=drawfeed))
			block.append("m5")

		def appendSpikes45(block):
			sinSpike = math.sin(math.atan(1.0 / math.sqrt(3)))
			cosSpike = math.cos(math.atan(1.0 / math.sqrt(3)))
			block.append("m5")
			block.append(CNC.grapid(x=x0, y=y0, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.gline(x=x0 + marksizehalf * sinSpike, y=y0 - marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 + marksizehalf * cosSpike, y=y0 - marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * cosSpike, y=y0 + marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * sinSpike, y=y0 + marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0, y=y0, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * sinSpike, y=y0 - marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 - marksizehalf * cosSpike, y=y0 - marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 + marksizehalf * cosSpike, y=y0 + marksizehalf * sinSpike, f=drawfeed))
			block.append(CNC.gline(x=x0 + marksizehalf * sinSpike, y=y0 + marksizehalf * cosSpike, f=drawfeed))
			block.append(CNC.gline(x=x0, y=y0, f=drawfeed))
			block.append("m5")
			block.append(CNC.grapid(x=x0 + marksizehalf / 2, y=y0 + marksizehalf / 2, f=movefeed))
			block.append(self.getPowerLine(app))
			block.append(CNC.gline(x=x0 - marksizehalf / 2, f=drawfeed))
			block.append(CNC.gline(y=y0 - marksizehalf / 2, f=drawfeed))
			block.append(CNC.gline(x=x0 + marksizehalf / 2, f=drawfeed))
			block.append(CNC.gline(y=y0 + marksizehalf / 2, f=drawfeed))
			block.append("m5")

		x0 = self.fromMm("PosX")
		y0 = self.fromMm("PosY")
		movefeed = app.cnc["cutfeed"]
		drawfeed = app.cnc["cutfeedz"]
		marksizehalf = self.fromMm("Mark size") / 2
		markpower = self["Mark power"]
		marktype = self["Mark type"]
		if "None" == marktype:
			pass
		else:
			block.append("m5")
			if "Star" == marktype:
				appendCross(block)
				appendCross45(block)
			if "Cross" == marktype:
				appendCross(block)
			if "Cross45" == marktype:
				appendCross45(block)
			if "Spikes" == marktype:
				appendSpikes(block)
			if "Spikes45" == marktype:
				appendSpikes45(block)
			if "SpikesStar" == marktype:
				appendSpikes(block)
				appendCross45(block)
			if "SpikesStar45" == marktype:
				appendSpikes45(block)
				appendCross(block)

			if self["Draw ring"]:
				appendCircle(block)
			else:
				block.append("m5")

	def execute(self, app):
		name = self["name"]
		if not name or name == "default":
			name = "Drillmark"
		marksize = self["Mark size"]
		x0 = self.fromMm("PosX")
		y0 = self.fromMm("PosY")
		marktype = self["Mark type"]
		block = Block(name + " %s diameter %s" % (marktype, CNC.fmt("", marksize)))
		self.appendBurn(app, block)
		self.appendMark(app, block)
		active = app.activeBlock()
		if active == 0:
			active = 1
		blocks = [block]
		app.gcode.insBlocks(active, blocks, _("Manual drill mark"))
		app.refresh()  # <<< refresh editor
		app.setStatus(_("Generated: Manual drillmark"))



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	Vasilis.Vlachoudis@cern.ch
# Date:	20-Aug-2015

from __future__ import absolute_import
from __future__ import print_function
__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

__name__ = _("Gear")

import math
from bmath import Vector
from CNC import CW,CNC,Block
from ToolsPage import Plugin


#==============================================================================
# Gear class
#==============================================================================
class Gear:
	def __init__(self, name): #, R, nteeth):
		self.name = name

	#----------------------------------------------------------------------
	def involute_intersect_angle(self, Rb, R):
		return math.sqrt(R*R - Rb*Rb) / Rb - math.acos(Rb/R)

	#----------------------------------------------------------------------
	def point_on_circle(self, radius, angle):
		return Vector(radius * math.cos(angle), radius * math.sin(angle))

	#----------------------------------------------------------------------
	# N   = no of teeth
	# phi = pressure angle
	# PC  = Circular Pitch
	#----------------------------------------------------------------------
	def calc(self, N, phi, Pc):
		N = abs(N)
		# Pitch Circle
		D = N * Pc / math.pi
		R = D / 2.0

		# Diametrical pitch
		Pd = N / D

		# Base Circle
		Db = D * math.cos(phi)
		Rb = Db / 2.0

		# Addendum
		a = 1.0 / Pd

		# Outside Circle
		Ro = R + a
		Do = 2.0 * Ro

		# Tooth thickness
		T = math.pi*D / (2*N)

		# undercut?
		U = 2.0 / (math.sin(phi) * (math.sin(phi)))
		needs_undercut = N < U
		# sys.stderr.write("N:%s R:%s Rb:%s\n" % (N,R,Rb))

		# Clearance
		c = 0.0
		# Dedendum
		b = a + c

		# Root Circle
		Rr = R - b
		Dr = 2.0*Rr

		two_pi = 2.0*math.pi
		half_thick_angle = two_pi / (4.0*N)
		pitch_to_base_angle = self.involute_intersect_angle(Rb, R)
		pitch_to_outer_angle = self.involute_intersect_angle(Rb, Ro) # pitch_to_base_angle

		points = []
		for x in range(1,N+1):
			c = x * two_pi / N

			# angles
			pitch1 = c - half_thick_angle
			base1  = pitch1 - pitch_to_base_angle
			outer1 = pitch1 + pitch_to_outer_angle

			pitch2 = c + half_thick_angle
			base2  = pitch2 + pitch_to_base_angle
			outer2 = pitch2 - pitch_to_outer_angle

			# points
			b1 = self.point_on_circle(Rb, base1)
			p1 = self.point_on_circle(R,  pitch1)
			o1 = self.point_on_circle(Ro, outer1)
			o2 = self.point_on_circle(Ro, outer2)
			p2 = self.point_on_circle(R,  pitch2)
			b2 = self.point_on_circle(Rb, base2)

			if Rr >= Rb:
				pitch_to_root_angle = pitch_to_base_angle - self.involute_intersect_angle(Rb, Rr)
				root1 = pitch1 - pitch_to_root_angle
				root2 = pitch2 + pitch_to_root_angle
				r1 = self.point_on_circle(Rr, root1)
				r2 = self.point_on_circle(Rr, root2)

				points.append(r1)
				points.append(p1)
				points.append(o1)
				points.append(o2)
				points.append(p2)
				points.append(r2)
			else:
				r1 = self.point_on_circle(Rr, base1)
				r2 = self.point_on_circle(Rr, base2)
				points.append(r1)
				points.append(b1)
				points.append(p1)
				points.append(o1)
				points.append(o2)
				points.append(p2)
				points.append(b2)
				points.append(r2)

		first = points[0]
		del points[0]

		blocks = []
		block = Block(self.name)
		blocks.append(block)

		block.append(CNC.grapid(first.x(), first.y()))
		block.append(CNC.zenter(0.0))
		#print first.x(), first.y()
		for v in points:
			block.append(CNC.gline(v.x(), v.y()))
			#print v.x(), v.y()
		#print first.x(), first.y()
		block.append(CNC.gline(first.x(), first.y()))
		block.append(CNC.zsafe())

		#block = Block("%s-center"%(self.name))
		block = Block("%s-basecircle"%(self.name))
		block.enable = False
		block.append(CNC.grapid(Db/2, 0.))
		block.append(CNC.zenter(0.0))
		block.append(CNC.garc(CW, Db/2, 0., i=-Db/2))
		block.append(CNC.zsafe())
		blocks.append(block)
		return blocks


#==============================================================================
# Create a simple Gear
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Generate a spur gear")

	def __init__(self, master):
		Plugin.__init__(self, master, "Gear")
		self.icon  = "gear"
		self.group = "Generator"
		self.variables = [
			("name",      "db",    "", _("Name")),
			("n",        "int",    10, _("No of teeth")),
			("phi",    "float",  17.0, _("Pressure angle")),
			("pc",        "mm",   5.0, _("Circular Pitch"))
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def execute(self, app):
		n = self["name"]
		if not n or n=="default": n="Gear"
		gear = Gear(n)
		blocks = gear.calc(self["n"], math.radians(self["phi"]), self.fromMm("pc"))
		active = app.activeBlock()
		if active==0: active=1
		app.gcode.insBlocks(active, blocks, _("Create Spur GEAR"))
		app.refresh()
		app.setStatus(_("Generated: Spur GEAR"))

if __name__=="__main__":
	gear = Gear()
	gear.calc(12, math.radians(10), math.radians(10))
	gear.calc(36, math.radians(10), math.radians(10))
#	gear.calc(10, math.radians(10), math.radians(10))
	#b:scale(Coord(0,0),Coord(10,10))
